
import numpy as np
import sympy as sym

def ac_3ph_4w_l(grid,vsc_data):
    '''
    VSC with 3 phase and 4 wire working in open loop as a grid former.
    
    '''

    params_dict  = grid.dae['params_dict']
    f_list = grid.dae['f']
    x_list = grid.dae['x']
    g_list = grid.dae['g'] 
    y_ini_list = grid.dae['y_ini'] 
    u_ini_dict = grid.dae['u_ini_dict']
    y_run_list = grid.dae['y_run'] 
    u_run_dict = grid.dae['u_run_dict']
    h_dict = grid.dae['h_dict']


    alpha = np.exp(2.0/3*np.pi*1j)
    A_0a =  np.array([[1, 1, 1],
                    [1, alpha**2, alpha],
                    [1, alpha, alpha**2]])

    A_a0 = 1/3* np.array([[1, 1, 1],
                        [1, alpha, alpha**2],
                        [1, alpha**2, alpha]])

    omega_coi_i = 0
    HS_coi = 0

    omega_coi = sym.Symbol('omega_coi',real=True)
    xi_freq = sym.Symbol('xi_freq',real=True)

    #vscs = [
    #    {'bus':'B1','S_n':100e3,'R':0.01,'X':0.1,'R_n':0.01,'X_n':0.1,'R_ng':0.01,'X_ng':3.0,'K_f':0.1,'T_f':1.0,'K_sec':0.5,'K_delta':0.001},
    #    ]

    #for vsc in vsc_data:
        
    name = vsc_data['bus']

    # inputs
    v_dc = sym.Symbol(f'v_dc_{name}', real=True)
    m_a,m_b,m_c,m_n = sym.symbols(f'm_a_{name},m_b_{name},m_c_{name},m_n_{name}', real=True)
    phi = sym.Symbol(f'phi_{name}', real=True)
    phi_a = sym.Symbol(f'phi_a_{name}', real=True)
    phi_b = sym.Symbol(f'phi_b_{name}', real=True)
    phi_c = sym.Symbol(f'phi_c_{name}', real=True)
    phi_n = sym.Symbol(f'phi_n_{name}', real=True)

    # parameters
    R_s,R_sn,R_ng = sym.symbols(f'R_{name}_s,R_{name}_sn,R_{name}_ng', real=True)
    X_s,X_sn,X_ng = sym.symbols(f'X_{name}_s,X_{name}_sn,X_{name}_ng', real=True)
    
    # dynamical states
    De_ao_m,De_bo_m,De_co_m,De_no_m  = sym.symbols(f'De_ao_m_{name},De_bo_m_{name},De_co_m_{name},De_no_m_{name}', real=True)

    omega = sym.Symbol(f'omega_{name}', real=True)
    
    ## algebraic states
    v_sa_r,v_sb_r,v_sc_r,v_sn_r,v_og_r = sym.symbols(f'V_{name}_0_r,V_{name}_1_r,V_{name}_2_r,V_{name}_3_r,v_{name}_o_r', real=True)
    v_sa_i,v_sb_i,v_sc_i,v_sn_i,v_og_i = sym.symbols(f'V_{name}_0_i,V_{name}_1_i,V_{name}_2_i,V_{name}_3_i,v_{name}_o_i', real=True)
    i_sa_r,i_sb_r,i_sc_r,i_sn_r,i_ng_r = sym.symbols(f'i_vsc_{name}_a_r,i_vsc_{name}_b_r,i_vsc_{name}_c_r,i_vsc_{name}_n_r,i_vsc_{name}_ng_r', real=True)
    i_sa_i,i_sb_i,i_sc_i,i_sn_i,i_ng_i = sym.symbols(f'i_vsc_{name}_a_i,i_vsc_{name}_b_i,i_vsc_{name}_c_i,i_vsc_{name}_n_i,i_vsc_{name}_ng_i', real=True)
    p_dc,i_dc = sym.symbols(f'p_dc_{name},i_dc_{name}', real=True)
    p_pos,q_pos = sym.symbols(f'p_pos_{name},q_pos_{name}', real=True)

    Z_sa = R_s + 1j*X_s
    Z_sb = R_s + 1j*X_s
    Z_sc = R_s + 1j*X_s
    Z_sn = R_sn + 1j*X_sn
    Z_ng = R_ng + 1j*X_ng

    i_sa = i_sa_r + 1j*i_sa_i
    i_sb = i_sb_r + 1j*i_sb_i
    i_sc = i_sc_r + 1j*i_sc_i
    i_sn = i_sn_r + 1j*i_sn_i

    v_sa = v_sa_r + 1j*v_sa_i
    v_sb = v_sb_r + 1j*v_sb_i
    v_sc = v_sc_r + 1j*v_sc_i
    v_sn = v_sn_r + 1j*v_sn_i
    v_og = v_og_r + 1j*v_og_i

    s_sa = v_sa * sym.conjugate(i_sa)
    s_sb = v_sb * sym.conjugate(i_sb)
    s_sc = v_sc * sym.conjugate(i_sc)
    s_sn = v_sn * sym.conjugate(i_sc)

    s_s_total = s_sa + s_sb + s_sc + s_sn
    p_s_total = sym.re(s_s_total) 

    sqrt6 = np.sqrt(6)
    e_ao_r = (m_a*v_dc/sqrt6)*sym.cos(phi_a + phi) 
    e_ao_i = (m_a*v_dc/sqrt6)*sym.sin(phi_a + phi) 
    e_bo_r = (m_b*v_dc/sqrt6)*sym.cos(phi_b + phi-2/3*np.pi) 
    e_bo_i = (m_b*v_dc/sqrt6)*sym.sin(phi_b + phi-2/3*np.pi) 
    e_co_r = (m_c*v_dc/sqrt6)*sym.cos(phi_c + phi-4/3*np.pi) 
    e_co_i = (m_c*v_dc/sqrt6)*sym.sin(phi_c + phi-4/3*np.pi) 
    e_no_r = (m_n*v_dc/sqrt6)*sym.cos(phi_n) 
    e_no_i = (m_n*v_dc/sqrt6)*sym.sin(phi_n) 

    e_ao_cplx = e_ao_r + 1j*e_ao_i
    e_bo_cplx = e_bo_r + 1j*e_bo_i
    e_co_cplx = e_co_r + 1j*e_co_i
    e_no_cplx = e_no_r + 1j*e_no_i

    s_ta = e_ao_cplx * sym.conjugate(i_sa)
    s_tb = e_bo_cplx * sym.conjugate(i_sb)
    s_tc = e_co_cplx * sym.conjugate(i_sc)
    s_tn = e_no_cplx * sym.conjugate(i_sc)

    p_ac_a = sym.re(s_ta)
    p_ac_b = sym.re(s_tb)
    p_ac_c = sym.re(s_tc)
    p_ac_n = sym.re(s_tn)

    v_san = v_sa - v_sn
    v_sbn = v_sb - v_sn
    v_scn = v_sc - v_sn

    eq_i_sa_cplx = v_og + e_ao_cplx - i_sa*Z_sa - v_sa   # v_sa = v_sag
    eq_i_sb_cplx = v_og + e_bo_cplx - i_sb*Z_sb - v_sb
    eq_i_sc_cplx = v_og + e_co_cplx - i_sc*Z_sc - v_sc
    eq_i_sn_cplx = v_og + e_no_cplx - i_sn*Z_sn - v_sn
    eq_v_og_cplx = i_sa + i_sb + i_sc + i_sn + v_og/Z_ng

    ### DC side
    A_loss = sym.symbols(f'A_loss_{name}',real=True)
    B_loss = sym.symbols(f'B_loss_{name}',real=True)
    C_loss = sym.symbols(f'C_loss_{name}',real=True)
    i_rms_a = sym.sqrt(i_sa_r**2+i_sa_i**2+1e-6) 
    i_rms_b = sym.sqrt(i_sb_r**2+i_sb_i**2+1e-6) 
    i_rms_c = sym.sqrt(i_sc_r**2+i_sc_i**2+1e-6) 
    i_rms_n = sym.sqrt(i_sn_r**2+i_sn_i**2+1e-6) 

    p_loss_a = A_loss*i_rms_a*i_rms_a + B_loss*i_rms_a + C_loss
    p_loss_b = A_loss*i_rms_b*i_rms_b + B_loss*i_rms_b + C_loss
    p_loss_c = A_loss*i_rms_c*i_rms_c + B_loss*i_rms_c + C_loss
    p_loss_n = A_loss*i_rms_n*i_rms_n + B_loss*i_rms_n + C_loss

    p_vsc_loss = p_loss_a + p_loss_b + p_loss_c + p_loss_n
    p_ac = p_ac_a + p_ac_b + p_ac_c + p_ac_n

    eq_p_dc = -p_dc + p_ac + p_vsc_loss
    eq_i_dc =  p_dc/v_dc - i_dc
 
    g_list += [sym.re(eq_i_sa_cplx)] 
    g_list += [sym.re(eq_i_sb_cplx)] 
    g_list += [sym.re(eq_i_sc_cplx)] 
    g_list += [sym.re(eq_i_sn_cplx)] 
    g_list += [sym.re(eq_v_og_cplx)] 
    g_list += [sym.im(eq_i_sa_cplx)] 
    g_list += [sym.im(eq_i_sb_cplx)] 
    g_list += [sym.im(eq_i_sc_cplx)] 
    g_list += [sym.im(eq_i_sn_cplx)] 
    g_list += [sym.im(eq_v_og_cplx)] 
    g_list += [eq_p_dc,eq_i_dc]

    y_ini_list += [i_sa_r,i_sb_r,i_sc_r,i_sn_r,v_og_r,p_dc,i_dc]
    y_ini_list += [i_sa_i,i_sb_i,i_sc_i,i_sn_i,v_og_i]
    y_run_list += [i_sa_r,i_sb_r,i_sc_r,i_sn_r,v_og_r,p_dc,i_dc]
    y_run_list += [i_sa_i,i_sb_i,i_sc_i,i_sn_i,v_og_i]

    for ph in ['a','b','c','n']:
        i_s_r = sym.Symbol(f'i_vsc_{name}_{ph}_r', real=True)
        i_s_i = sym.Symbol(f'i_vsc_{name}_{ph}_i', real=True)  
        idx_r,idx_i = grid.node2idx(name,ph)
        grid.dae['g'] [idx_r] += -i_s_r
        grid.dae['g'] [idx_i] += -i_s_i
        i_s = i_s_r + 1j*i_s_i
        i_s_m = np.abs(i_s)
        h_dict.update({f'i_vsc_{name}_{ph}_m':i_s_m})

    u_ini_dict.update({f'v_dc_{name}':800.0})
    u_run_dict.update({f'v_dc_{name}':800.0})
    m = 0.7071
    u_ini_dict.update({f'm_a_{name}':m,
                       f'm_b_{name}':m,
                       f'm_c_{name}':m,
                       f'm_n_{name}':0.0})
    u_run_dict.update({f'm_a_{name}':m,
                       f'm_b_{name}':m,
                       f'm_c_{name}':m,
                       f'm_n_{name}':0.0})

    u_ini_dict.update({f'phi_{name}':0.0})
    u_ini_dict.update({f'phi_a_{name}':0.0})
    u_ini_dict.update({f'phi_b_{name}':0.0})
    u_ini_dict.update({f'phi_c_{name}':0.0})
    u_ini_dict.update({f'phi_n_{name}':0.0})

    u_run_dict.update({f'phi_{name}':0.0})
    u_run_dict.update({f'phi_a_{name}':0.0})
    u_run_dict.update({f'phi_b_{name}':0.0})
    u_run_dict.update({f'phi_c_{name}':0.0})
    u_run_dict.update({f'phi_n_{name}':0.0})

    params_dict.update({f'X_{name}_s':vsc_data['X'],f'R_{name}_s':vsc_data['R']})
    params_dict.update({f'X_{name}_sn':vsc_data['X_n'],f'R_{name}_sn':vsc_data['R_n']})
    params_dict.update({f'X_{name}_ng':vsc_data['X_ng'],f'R_{name}_ng':vsc_data['R_ng']})
    
    params_dict.update({f'S_n_{name}':vsc_data['S_n']})
   
    v_sabc = sym.Matrix([[v_sa],[v_sb],[v_sc]])
    i_sabc = sym.Matrix([[i_sa],[i_sb],[i_sc]])
    
    v_szpn = A_a0*v_sabc
    i_szpn = A_a0*i_sabc
    
    s_pos = 3*v_szpn[1]*sym.conjugate(i_szpn[1])
    s_neg = 3*v_szpn[2]*sym.conjugate(i_szpn[2])
    s_zer = 3*v_szpn[0]*sym.conjugate(i_szpn[0])

    g_list += [sym.re(s_pos) - p_pos]
    g_list += [sym.im(s_pos) - q_pos]

    y_ini_list += [p_pos,q_pos]
    y_run_list += [p_pos,q_pos]

    h_dict.update({f'p_{name}_pos':sym.re(s_pos),f'p_{name}_neg':sym.re(s_neg),f'p_{name}_zer':sym.re(s_zer)})
    h_dict.update({str(m_a):m_a,str(m_b):m_b,str(m_c):m_c,str(m_n):m_n})
    h_dict.update({str(phi):phi})
    h_dict.update({f'p_dc_{name}':p_dc})
    h_dict.update({f'p_s_total_{name}':p_s_total})
    
    
    S_n_num = vsc_data['S_n']
    U_n_num = vsc_data['U_n']
    I_n = S_n_num/(np.sqrt(3)*U_n_num)
    P_0 = 0.01*S_n_num
    C_loss_num = P_0/3
    P_cc = 0.01*S_n_num
    A_loss_num = P_cc/(I_n**2)/3
    B_loss_num = 1.0

    if 'A_loss' in vsc_data:
        A_loss_num = vsc_data['A_loss']
        B_loss_num = vsc_data['B_loss']
        C_loss_num = vsc_data['C_loss']

    params_dict.update({str(A_loss):A_loss_num})
    params_dict.update({str(B_loss):B_loss_num})
    params_dict.update({str(C_loss):C_loss_num})


def test():
    import numpy as np
    import sympy as sym
    import json
    from pydae.urisi.urisi_builder import urisi
    import pydae.build_cffi as db

    grid = urisi('ac_3ph_4w_l.hjson')
    grid.construct('temp')
    grid.compile('temp')

    import temp

    S_n = 100e3
    V_n = 400
    I_n = S_n/V_n
    Conduction_losses = 0.02*S_n # = A*I_n**2
    lossses = 1.0
    A = Conduction_losses/(I_n**2)/3*lossses
    B = 1/3*lossses
    C = 0.02*S_n/3*lossses
    model = temp.model()
    m = 0.75
    phi = 0.1
    model.ini({
               #'A_loss_A1':A,'B_loss_A1':B,'C_loss_A1':C,
               'm_a_A1':m,'m_b_A1':m,'m_c_A1':m,
               'phi_a_A1':phi,'phi_b_A1':phi,'phi_c_A1':phi,
               },'xy_0.json')
    model.report_y()
    model.report_z()


if __name__ == '__main__':

    #development()
    test()



# -*- coding: utf-8 -*-
"""
Created on Thu August 10 23:52:55 2022

@author: jmmauricio
"""

import numpy as np
import sympy as sym


def add_shunts(self):

    for shunt in self.shunts:
        node_j_str = str(shunt['bus_nodes'][0])
        node_j = '{:s}.{:s}'.format(shunt['bus'], node_j_str)
        col = self.nodes_list.index(node_j)           
        row_j = self.it_branch
        self.A[row_j,col] = 1
        
        #node_k_str = str(shunt['bus_nodes'][1])
        #if not node_k_str == '0': # when connected to ground
        #    node_k = '{:s}.{:s}'.format(shunt['bus'], str(shunt['bus_nodes'][1]))
        #    row_k = self.nodes_list.index(node_k)            
        #    self.A[row_k,col] = -1
        shunt_name = f"shunt_{shunt['bus']}_{node_j_str}"
        g_jk = sym.Symbol(f"g_{shunt_name}", real=True) 
        b_jk = sym.Symbol(f"b_{shunt_name}", real=True) 
        self.G_primitive[self.it_branch,self.it_branch] = g_jk
        self.B_primitive[self.it_branch,self.it_branch] = b_jk

        Z = shunt['R'] + 1j*shunt['X']
        Y = 1/Z
        self.dae['params_dict'].update({str(g_jk):Y.real})
        self.dae['params_dict'].update({str(b_jk):Y.imag})
       
        self.it_branch += 1



def shunts_preprocess(self):

    for shunt in self.shunts:
        
        self.N_branches += 1

        


import sympy as sym
import numpy as np
from sympy import interpolating_spline


def bess_dcdc(grid,data):
    '''

    PV+DCDC model MPPT control is implemented. 

    parameters
    ----------

    S_n: nominal power in VA
    U_n: nominal rms phase to phase voltage in V
    F_n: nominal frequency in Hz
    X_s: coupling reactance in pu (base machine S_n)
    R_s: coupling resistance in pu (base machine S_n)

    inputs
    ------

    p_s_ref: active power reference (pu, S_n base)
    q_s_ref: reactive power reference (pu, S_n base)
    v_dc: dc voltage in pu (when v_dc = 1 and m = 1, v_ac = 1)

    example
    -------

    "vscs": [{"bus":bus_name,"type":"pv_mpt_dcdc",
                 "S_n":1e6,"U_n":400.0,"F_n":50.0,
                 "X_s":0.1,"R_s":0.01,"monitor":True,
                 "I_sc":3.87,"V_oc":42.1,"I_mp":3.56,"V_mp":33.7,
                 "K_vt":-0.160,"K_it":0.065,
                 "N_pv_s":25,"N_pv_p":250}]
    
    '''
    
    bus_name = data['bus']
    name = bus_name

    # commons
    ## voltages:
    v_hp, v_hn = sym.symbols(f'V_{bus_name}_0_r,V_{bus_name}_1_r', real=True)
    v_hp_i, v_hn_i = sym.symbols(f'V_{bus_name}_0_i,V_{bus_name}_1_i', real=True)
   
    ## algebraic states
    i_h = sym.Symbol(f"i_h_{name}", real=True)
    p_l = sym.Symbol(f"p_l_{name}", real=True)

    ### HV-side
    e_h,v_h,i_h,p_h = sym.symbols(f'e_h_{name},v_h_{name},i_h_{name},p_h_{name}',real=True)
    R_h = sym.Symbol(f'R_h_{name}',real=True)
    p_r_ref = sym.Symbol(f'p_r_ref_{name}',real=True)
    m_h = sym.Symbol(f'm_h_{name}',real=True)
    i_g = sym.Symbol(f'i_g_{name}',real=True) 
    R_g = sym.Symbol(f'R_g_{name}',real=True)

    # dynamic states
    soc = sym.Symbol(f"soc_{name}", real=True)
    xi_soc = sym.Symbol(f"xi_soc_{name}", real=True)

    # algebraic states
    v_dc = sym.Symbol(f"v_dc_{name}", real=True)


    # Battery
    E_kWh = sym.Symbol(f"E_kWh_{name}", real=True)
    soc_min = sym.Symbol(f"soc_min_{name}", real=True)
    soc_max = sym.Symbol(f"soc_max_{name}", real=True)
    A_loss = sym.Symbol(f"A_loss_{name}", real=True)
    B_loss = sym.Symbol(f"B_loss_{name}", real=True)
    C_loss = sym.Symbol(f"C_loss_{name}", real=True)
    K_p = sym.Symbol(f"K_p_{name}", real=True)
    K_i = sym.Symbol(f"K_i_{name}", real=True)
    B_0 = sym.Symbol(f"B_0_{name}", real=True)
    B_1 = sym.Symbol(f"B_1_{name}", real=True)
    B_2 = sym.Symbol(f"B_2_{name}", real=True)
    B_3 = sym.Symbol(f"B_3_{name}", real=True)
    R_bat = sym.Symbol(f"R_bat_{name}", real=True)
    soc_ref = sym.Symbol(f"soc_ref_{name}", real=True) 

    epsilon_soc = soc_ref - soc
    p_soc = -(K_p*epsilon_soc + K_i*xi_soc)
    E_n = 1000*3600*E_kWh

    if 'socs' in data:
        socs = np.array(data['socs'])
        es = np.array(data['es'])
        interpolation = interpolating_spline(1, soc, socs, es)
        interpolation._args = tuple(list(interpolation._args) + [sym.functions.elementary.piecewise.ExprCondPair(0,True)])
        e = interpolation
    else:
        e = B_0 + B_1*soc + B_2*soc**2 + B_3*soc**3
    i_dc = p_l/v_dc

    ## dynamic equations    
    dsoc = 1/E_n*(-i_dc*e)   
    dxi_soc = epsilon_soc     

    ## algebraic equations   
    g_v_dc = e - i_dc*R_bat - v_dc

    grid.dae['f'] += [dsoc, dxi_soc]
    grid.dae['x'] += [ soc,  xi_soc] 

    grid.dae['g'] +=     [g_v_dc]
    grid.dae['y_ini'] += [  v_dc]  
    grid.dae['y_run'] += [  v_dc]  

    soc_ref_N = 0.5
    if 'soc_0' in data:
        soc_ref_N = data['soc_0']

    grid.dae['u_ini_dict'].update({f'{str(soc_ref)}':soc_ref_N})
    grid.dae['u_run_dict'].update({f'{str(soc_ref)}':soc_ref_N})

    ## parameters  
    grid.dae['params_dict'].update({f"{K_p}":1e-6}) 
    grid.dae['params_dict'].update({f"{K_i}":1e-6})  
    grid.dae['params_dict'].update({f"{soc_min}":0.0}) 
    grid.dae['params_dict'].update({f"{soc_max}":1.0})           
    grid.dae['params_dict'].update({f"{E_kWh}":data['E_kWh']}) 

    if 'B_0' in data:
        B_0_N = data['B_0_N']
        B_1_N = data['B_1_N']
        B_2_N = data['B_2_N']
        B_3_N = data['B_3_N']
    else:
        B_0_N = 600
        B_1_N = 0.0
        B_2_N = 0.0
        B_3_N = 0.0

    grid.dae['params_dict'].update({f"{B_0}":B_0_N}) 
    grid.dae['params_dict'].update({f"{B_1}":B_1_N}) 
    grid.dae['params_dict'].update({f"{B_2}":B_2_N}) 
    grid.dae['params_dict'].update({f"{B_3}":B_3_N}) 

    if 'R_bat' in data:
        R_bat_N = data['R_bat']
    else:
        R_bat_N = 0.0

    grid.dae['params_dict'].update({f"{R_bat}":R_bat_N}) 

    grid.dae['xy_0_dict'].update({f"v_dc_{name}":B_0_N})


    # VSC

    ## inputs
    p_r_ref = sym.Symbol(f"p_r_ref_{name}", real=True)
    
    ## algebraic states
  

    p_ref = sym.Piecewise((p_r_ref,(p_r_ref <=0.0) & (soc<soc_max)),
                        (p_r_ref,(p_r_ref > 0.0) & (soc>soc_min)),
                        (0.0,True)) + p_soc
    
    v_l = v_dc
    p_h = p_ref
    v_h = v_hp - v_hn
    i_hp =  i_h
    i_hn = -i_h
    p_loss = A_loss*i_hp**2 + B_loss*sym.Abs(i_hp) + C_loss

    eq_i_h = -v_h*i_h + p_h
    eq_p_l = -p_l + p_h + p_loss

    grid.dae['g'] += [eq_i_h, eq_p_l]
    grid.dae['y_ini'] += [ i_h, p_l]  
    grid.dae['y_run'] += [ i_h, p_l]  

    grid.dae['u_ini_dict'].update({f'{str(p_r_ref)}':0.0})
    grid.dae['u_run_dict'].update({f'{str(p_r_ref)}':0.0})

    ## current injections dc LV side
    idx_hp_r,idx_hp_i = grid.node2idx(f'{bus_name}','a')
    idx_hn_r,idx_hn_i = grid.node2idx(f'{bus_name}','b')
    grid.dae['g'] [idx_hp_r] += -i_hp 
    grid.dae['g'] [idx_hn_r] += -i_hn
    grid.dae['g'] [idx_hp_i] +=  v_hp_i/1e3
    grid.dae['g'] [idx_hn_i] +=  v_hn_i/1e3
   
    if 'A_loss' in data:
        A_loss_N = data['A_loss']
        B_loss_N = data['B_loss']
        C_loss_N = data['C_loss']
    else:
        A_loss_N = 0.0001
        B_loss_N = 0.0
        C_loss_N = 0.0001

    grid.dae['params_dict'].update({f"{str(A_loss)}":A_loss_N}) 
    grid.dae['params_dict'].update({f"{str(B_loss)}":B_loss_N}) 
    grid.dae['params_dict'].update({f"{str(C_loss)}":C_loss_N}) 

    grid.dae['params_dict'].update({f"{E_kWh}":data['E_kWh']}) 

    grid.dae['xy_0_dict'].update({str(v_dc):B_0_N})
    grid.dae['xy_0_dict'].update({str(soc):0.5})
    grid.dae['xy_0_dict'].update({str(xi_soc):0.5})

    ## outputs
    grid.dae['h_dict'].update({f'p_loss_{name}':p_loss})
    grid.dae['h_dict'].update({f'i_dc_{name}':i_dc})
    grid.dae['h_dict'].update({f'p_h_{name}':p_h})

def test():
    import numpy as np
    import sympy as sym
    import json
    from pydae.urisi.urisi_builder import urisi
    import pydae.build_cffi as db

    grid = urisi('bess_dcdc.hjson')
    grid.uz_jacs = True
    grid.construct('temp')
    grid.compile('temp')

    import temp

    S_n = 500e3
    V_n = 800
    I_n = S_n/V_n
    Conduction_losses = 0.02*S_n # = A*I_n**2
    losses = 1.0
    A = Conduction_losses/(I_n**2)*losses
    B = 1*losses
    C = 0.02*S_n*losses
    model = temp.model()
    model.ini({'A_loss_D1':A,'B_loss_D1':B,'C_loss_D1':C},'xy_0.json')
    #model.report_params()
    model.report_u()
    model.report_y()
    model.report_z()



if __name__ == '__main__':

    #development()
    test()






import numpy as np
import sympy as sym



def ac3ph4wgfpidq3(grid,vsc_data):
    '''
    VSC with 3 phase and 4 wire working in open loop as a grid former.

    
    '''

    sin = sym.sin
    cos = sym.cos
    sqrt = sym.sqrt
    exp = sym.exp
    
    params_dict  = grid.dae['params']
    f_list = grid.dae['f']
    x_list = grid.dae['x']
    g_list = grid.dae['g'] 
    y_list = grid.dae['y'] 
    u_dict = grid.dae['u']
    h_dict = grid.dae['h_dict']


    alpha = np.exp(2.0/3*np.pi*1j)
    A_0a =  np.array([[1, 1, 1],
                    [1, alpha**2, alpha],
                    [1, alpha, alpha**2]])

    A_a0 = 1/3* np.array([[1, 1, 1],
                        [1, alpha, alpha**2],
                        [1, alpha**2, alpha]])

    omega_coi_i = 0
    HS_coi = 0

    omega_coi = sym.Symbol('omega_coi',real=True)
    xi_freq = sym.Symbol('xi_freq',real=True)
    K_agc = sym.Symbol('K_agc',real=True)

    #vscs = [
    #    {'bus':'B1','S_n':100e3,'R':0.01,'X':0.1,'R_n':0.01,'X_n':0.1,'R_ng':0.01,'X_ng':3.0,'K_f':0.1,'T_f':1.0,'K_sec':0.5,'K_delta':0.001},
    #    ]

    #for vsc in vsc_data:
    name = vsc_data['bus']

    # inputs
    e_ref_a_m,e_ref_b_m,e_ref_c_m = sym.symbols(f'e_ref_a_m_{name},e_ref_b_m_{name},e_ref_c_m_{name}', real=True)
    phi_ref_a,phi_ref_b,phi_ref_c = sym.symbols(f'phi_ref_a_{name},phi_ref_b_{name},phi_ref_c_{name}', real=True)
    p_c = sym.Symbol(f'p_c_{name}', real=True) 

    # parameters
    S_n,U_n,H,K_f,T_f,K_sec,K_delta  = sym.symbols(f'S_n_{name},U_n_{name},H_{name},K_f_{name},T_f_{name},K_sec_{name},K_delta_{name}', real=True)
    R_s,R_sn,R_ng = sym.symbols(f'R_{name}_s,R_{name}_sn,R_{name}_ng', real=True)
    X_s,X_sn,X_ng = sym.symbols(f'X_{name}_s,X_{name}_sn,X_{name}_ng', real=True)
    K_p = sym.Symbol(f"K_p_{name}", real=True)    
    T_p = sym.Symbol(f"T_p_{name}", real=True)   
    R_v = sym.Symbol(f"R_v_{name}", real=True)   
    X_v = sym.Symbol(f"X_v_{name}", real=True)   
    Droop = sym.Symbol(f"Droop_{name}", real=True)  
    T_wf = sym.Symbol(f"T_wf_{name}", real=True)
    T_sf = sym.Symbol(f"T_sf_{name}", real=True)

    # basis for per unit
    S_b = S_n
    U_b = U_n
    I_b = S_b/(np.sqrt(3)*U_b)
    U_bdq = U_b*(np.sqrt(2/3))
    I_bdq = I_b*np.sqrt(2)  
    
    # dynamical states
    phi = sym.Symbol(f'phi_{name}', real=True)
    xi_p = sym.Symbol(f'xi_p_{name}', real=True)
    omega_f = sym.Symbol(f'omega_f_{name}', real=True)
    p_s_f = sym.Symbol(f'p_s_f_{name}', real=True) 


    # algebraic states
    v_sa_r,v_sb_r,v_sc_r,v_sn_r,v_og_r = sym.symbols(f'v_{name}_a_r,v_{name}_b_r,v_{name}_c_r,v_{name}_n_r,v_{name}_o_r', real=True)
    v_sa_i,v_sb_i,v_sc_i,v_sn_i,v_og_i = sym.symbols(f'v_{name}_a_i,v_{name}_b_i,v_{name}_c_i,v_{name}_n_i,v_{name}_o_i', real=True)
    i_sa_r,i_sb_r,i_sc_r,i_sn_r,i_ng_r = sym.symbols(f'i_vsc_{name}_a_r,i_vsc_{name}_b_r,i_vsc_{name}_c_r,i_vsc_{name}_n_r,i_vsc_{name}_ng_r', real=True)
    i_sa_i,i_sb_i,i_sc_i,i_sn_i,i_ng_i = sym.symbols(f'i_vsc_{name}_a_i,i_vsc_{name}_b_i,i_vsc_{name}_c_i,i_vsc_{name}_n_i,i_vsc_{name}_ng_i', real=True)

    Z_sa = R_s + 1j*X_s
    Z_sb = R_s + 1j*X_s
    Z_sc = R_s + 1j*X_s
    Z_sn = R_sn + 1j*X_sn
    Z_ng = R_ng + 1j*X_ng

    i_sa = i_sa_r + 1j*i_sa_i
    i_sb = i_sb_r + 1j*i_sb_i
    i_sc = i_sc_r + 1j*i_sc_i
    i_sn = i_sn_r + 1j*i_sn_i

    v_sa = v_sa_r + 1j*v_sa_i
    v_sb = v_sb_r + 1j*v_sb_i
    v_sc = v_sc_r + 1j*v_sc_i
    v_sn = v_sn_r + 1j*v_sn_i
    v_og = v_og_r + 1j*v_og_i
    

    # Controller 

    s_s = 1/S_b*(v_sa * sym.conjugate(i_sa) + 
                 v_sb * sym.conjugate(i_sb) + 
                 v_sc * sym.conjugate(i_sc))
    p_s = sym.re(s_s)

    
    #references pu
    p_pri = 1.0/Droop*(1.0 - omega_f)
    p_agc = K_agc*xi_freq

    p_sec = K_sec*p_agc

    p_m_ref = p_sec + p_pri + p_c

    #error
    #epsilon_p = (p_m_ref - p_s);
    #error. Low pass filter or Notch filter
    epsilon_p = (p_m_ref - p_s_f)

    #omega VSG
    omega = K_p*(epsilon_p + xi_p/T_p) + 1.0; 

    #derivatives
    dphi =  2.0*np.pi*50.0*(omega - omega_coi) - K_delta*phi
    dxi_p = epsilon_p
    dp_s_f = 1.0/T_sf*(p_s - p_s_f)    #Low pass filter for p_s
    domega_f = 1.0/T_wf*(omega - omega_f)

    omega_rads = 2.0*np.pi*50.0*omega
    # from derivatives to the integrator
    h_dict.update({f'omega_{name}':omega})

    phi_vsg = phi
    
    i_sd_a = 1.0/I_b*(i_sa_i*cos(phi_vsg) + i_sa_r*sin(phi_vsg))
    i_sq_a = 1.0/I_b*(i_sa_i*sin(phi_vsg) - i_sa_r*cos(phi_vsg))
    i_sd_b = 1.0/I_b*(i_sb_i*cos(phi_vsg) + i_sb_r*sin(phi_vsg))
    i_sq_b = 1.0/I_b*(i_sb_i*sin(phi_vsg) - i_sb_r*cos(phi_vsg))
    i_sd_c = 1.0/I_b*(i_sc_i*cos(phi_vsg) + i_sc_r*sin(phi_vsg))
    i_sq_c = 1.0/I_b*(i_sc_i*sin(phi_vsg) - i_sc_r*cos(phi_vsg))
    
    h_dict.update({f'i_sd_a_{name}':i_sd_a,f'i_sq_a_{name}':i_sq_a})
    h_dict.update({f'i_sd_b_{name}':i_sd_b,f'i_sq_b_{name}':i_sq_b})
    h_dict.update({f'i_sd_c_{name}':i_sd_c,f'i_sq_c_{name}':i_sq_c})


    #v_sd_a = sqrt(2.0)/U_bdq*(v_sa_i*cos(phi_vsg) + v_sa_r*sin(phi_vsg))
    #v_sq_a = sqrt(2.0)/U_bdq*(v_sa_i*sin(phi_vsg) - v_sa_r*cos(phi_vsg))
    #v_sd_b = sqrt(2.0)/U_bdq*(v_sb_i*cos(phi_vsg) + v_sb_r*sin(phi_vsg))
    #v_sq_b = sqrt(2.0)/U_bdq*(v_sb_i*sin(phi_vsg) - v_sb_r*cos(phi_vsg))
    #v_sd_c = sqrt(2.0)/U_bdq*(v_sc_i*cos(phi_vsg) + v_sc_r*sin(phi_vsg))
    #v_sq_c = sqrt(2.0)/U_bdq*(v_sc_i*sin(phi_vsg) - v_sc_r*cos(phi_vsg))

    e_ref_a = e_ref_a_m*exp(sym.I*(phi_ref_a))
    e_ref_b = e_ref_b_m*exp(sym.I*(phi_ref_b))
    e_ref_c = e_ref_c_m*exp(sym.I*(phi_ref_c))

    e_ref_a_r = sym.re(e_ref_a)
    e_ref_a_i = sym.im(e_ref_a)
    e_ref_b_r = sym.re(e_ref_b)
    e_ref_b_i = sym.im(e_ref_b)
    e_ref_c_r = sym.re(e_ref_c)
    e_ref_c_i = sym.im(e_ref_c)

    e_d_ref_a = (e_ref_a_i*cos(phi_vsg) + e_ref_a_r*sin(phi_vsg))
    e_q_ref_a = (e_ref_a_i*sin(phi_vsg) - e_ref_a_r*cos(phi_vsg))
    e_d_ref_b = (e_ref_b_i*cos(phi_vsg) + e_ref_b_r*sin(phi_vsg))
    e_q_ref_b = (e_ref_b_i*sin(phi_vsg) - e_ref_b_r*cos(phi_vsg))
    e_d_ref_c = (e_ref_c_i*cos(phi_vsg) + e_ref_c_r*sin(phi_vsg))
    e_q_ref_c = (e_ref_c_i*sin(phi_vsg) - e_ref_c_r*cos(phi_vsg))

    #Voltage at the VSC terminal phase a
    v_t_d_a = -(i_sd_a*(R_v) - (X_v)*i_sq_a) + e_d_ref_a;  #pu
    v_t_q_a = -(i_sq_a*(R_v) + (X_v)*i_sd_a) + e_q_ref_a;  #pu

    #Voltage at the VSC terminal phase b
    v_t_d_b = -(i_sd_b*(R_v) - (X_v)*i_sq_b) + e_d_ref_b;  #pu
    v_t_q_b = -(i_sq_b*(R_v) + (X_v)*i_sd_b) + e_q_ref_b;  #pu

    #Voltage at the VSC terminal phase c
    v_t_d_c = -(i_sd_c*(R_v) - (X_v)*i_sq_c) + e_d_ref_c;  #pu
    v_t_q_c = -(i_sq_c*(R_v) + (X_v)*i_sd_c) + e_q_ref_c;  #pu

    v_t_a_i = U_b/np.sqrt(3.0)*( v_t_d_a*cos(phi_vsg) - v_t_q_a*sin(phi_vsg))
    v_t_a_r = U_b/np.sqrt(3.0)*(-v_t_d_a*sin(phi_vsg) - v_t_q_a*cos(phi_vsg))
    v_t_b_i = U_b/np.sqrt(3.0)*( v_t_d_b*cos(phi_vsg) - v_t_q_b*sin(phi_vsg))
    v_t_b_r = U_b/np.sqrt(3.0)*(-v_t_d_b*sin(phi_vsg) - v_t_q_b*cos(phi_vsg))
    v_t_c_i = U_b/np.sqrt(3.0)*( v_t_d_c*cos(phi_vsg) - v_t_q_c*sin(phi_vsg))
    v_t_c_r = U_b/np.sqrt(3.0)*(-v_t_d_c*sin(phi_vsg) - v_t_q_c*cos(phi_vsg))

    v_t_a = v_t_a_r + 1j*v_t_a_i
    v_t_b = v_t_b_r + 1j*v_t_b_i
    v_t_c = v_t_c_r + 1j*v_t_c_i

    h_dict.update({f'v_t_a_r_{name}':v_t_a_r,f'v_t_a_i_{name}':v_t_a_i})
    h_dict.update({f'v_t_b_r_{name}':v_t_b_r,f'v_t_b_i_{name}':v_t_b_i})
    h_dict.update({f'v_t_c_r_{name}':v_t_c_r,f'v_t_c_i_{name}':v_t_c_i})
    
    h_dict.update({f'v_t_d_a_{name}':v_t_d_a,f'v_t_q_a_{name}':v_t_q_a})
    h_dict.update({f'v_t_d_b_{name}':v_t_d_b,f'v_t_q_b_{name}':v_t_q_b})
    h_dict.update({f'v_t_d_c_{name}':v_t_d_c,f'v_t_q_c_{name}':v_t_q_c})   
    
    h_dict.update({f'e_d_ref_a_{name}':e_d_ref_a,f'e_q_ref_a_{name}':e_q_ref_a})
    h_dict.update({f'e_d_ref_b_{name}':e_d_ref_b,f'e_q_ref_b_{name}':e_q_ref_b})
    h_dict.update({f'e_d_ref_c_{name}':e_d_ref_c,f'e_q_ref_c_{name}':e_q_ref_c})


    ### VSC 4 wire:
    e_no_cplx = 0.0
    eq_i_sa_cplx = v_og + v_t_a - i_sa*Z_sa - v_sa   # v_sa = v_sag
    eq_i_sb_cplx = v_og + v_t_b - i_sb*Z_sb - v_sb
    eq_i_sc_cplx = v_og + v_t_c - i_sc*Z_sc - v_sc
    eq_i_sn_cplx = v_og + e_no_cplx - i_sn*Z_sn - v_sn
    eq_v_og_cplx = i_sa + i_sb + i_sc + i_sn + v_og/Z_ng

    g_list += [sym.re(eq_i_sa_cplx)] 
    g_list += [sym.re(eq_i_sb_cplx)] 
    g_list += [sym.re(eq_i_sc_cplx)] 
    g_list += [sym.re(eq_i_sn_cplx)] 
    g_list += [sym.re(eq_v_og_cplx)] 
    g_list += [sym.im(eq_i_sa_cplx)] 
    g_list += [sym.im(eq_i_sb_cplx)] 
    g_list += [sym.im(eq_i_sc_cplx)] 
    g_list += [sym.im(eq_i_sn_cplx)] 
    g_list += [sym.im(eq_v_og_cplx)] 

    y_list += [i_sa_r,i_sb_r,i_sc_r,i_sn_r,v_og_r]
    y_list += [i_sa_i,i_sb_i,i_sc_i,i_sn_i,v_og_i]

    y_ini_str = [str(item) for item in y_list]

    for ph in ['a','b','c','n']:
        i_s_r = sym.Symbol(f'i_vsc_{name}_{ph}_r', real=True)
        i_s_i = sym.Symbol(f'i_vsc_{name}_{ph}_i', real=True)  
        g_list[y_ini_str.index(f'v_{name}_{ph}_r')] += i_s_r
        g_list[y_ini_str.index(f'v_{name}_{ph}_i')] += i_s_i
        i_s = i_s_r + 1j*i_s_i
        i_s_m = np.abs(i_s)
        h_dict.update({f'i_vsc_{name}_{ph}_m':i_s_m})
    ### end VSC 4 wire
    
    #h_dict.update({f'v_t_a_{name}_r':sym.re(v_t_a),f'v_tb_{name}_r':sym.re(v_t_b),f'v_t_c_{name}_r':sym.re(v_t_c)})
        

    u_dict.update({f'e_ref_a_m_{name}':1.0,
                   f'e_ref_b_m_{name}':1.0,
                   f'e_ref_c_m_{name}':1.0,
                   f'e_{name}_no_m':0.0,
                   f'p_c_{name}':0.0})
    #u_dict.update({f'phi_{name}':0.0})
    u_dict.update({f'phi_ref_a_{name}': 0.0})
    u_dict.update({f'phi_ref_b_{name}':-2./3*np.pi})
    u_dict.update({f'phi_ref_c_{name}':-4./3*np.pi})

    u_dict.update({f'omega_{name}_ref':1.0})



    #for ph in ['a','b','c','n']:
    #    u_dict.pop(f'i_{name}_{ph}_r')
    #    u_dict.pop(f'i_{name}_{ph}_i')
    params_dict.update({f'X_v_{name}':vsc_data['X_v'],f'R_v_{name}':vsc_data['R_v']})
    params_dict.update({f'X_{name}_s':vsc_data['X'],f'R_{name}_s':vsc_data['R']})
    params_dict.update({f'X_{name}_sn':vsc_data['X_n'],f'R_{name}_sn':vsc_data['R_n']})
    params_dict.update({f'X_{name}_ng':vsc_data['X_ng'],f'R_{name}_ng':vsc_data['R_ng']})
    
    params_dict.update({f'S_n_{name}':vsc_data['S_n'],f'U_n_{name}':vsc_data['U_n']})
    params_dict.update({f'T_e_{name}':vsc_data['T_e']})
    params_dict.update({f'T_c_{name}':vsc_data['T_c']})
    params_dict.update({f'Droop_{name}':vsc_data['Droop']})
    params_dict.update({f'K_p_{name}':vsc_data['K_p']})
    params_dict.update({f'T_p_{name}':vsc_data['T_p']})
    params_dict.update({f'K_sec_{name}':vsc_data['K_sec']})
    params_dict.update({f'K_delta_{name}':vsc_data['K_delta']})
    params_dict.update({f'T_wf_{name}':0.1})
    params_dict.update({f'T_sf_{name}':0.1})
 
    
    v_sabc = sym.Matrix([[v_sa],[v_sb],[v_sc]])
    i_sabc = sym.Matrix([[i_sa],[i_sb],[i_sc]])
    
    v_szpn = A_a0*v_sabc
    i_szpn = A_a0*i_sabc
    
    s_pos = 3*v_szpn[1]*sym.conjugate(i_szpn[1])
    s_neg = 3*v_szpn[2]*sym.conjugate(i_szpn[2])
    s_zer = 3*v_szpn[0]*sym.conjugate(i_szpn[0])    

    f_list += [dphi,dxi_p,dp_s_f,domega_f]
    x_list += [ phi, xi_p, p_s_f, omega_f]
    
    #h_dict.update({f'p_{name}_pos':sym.re(s_pos),f'p_{name}_neg':sym.re(s_neg),f'p_{name}_zer':sym.re(s_zer)})
    #h_dict.update({str(p_c):p_c,str(omega_ref):omega_ref})
    #h_dict.update({f'p_pri_{name}':p_pri})

    HS_coi  = S_n
    omega_coi_i = S_n*omega

    grid.omega_coi_h_i += omega_coi_i
    grid.hs_total += HS_coi    
    
    u_dict.update({f'omega_{name}_ref':1.0})

    #for ph in ['a','b','c','n']:
    #    u_dict.pop(f'i_{name}_{ph}_r')
    #    u_dict.pop(f'i_{name}_{ph}_i')
    params_dict.update({f'X_v_{name}':vsc_data['X_v'],f'R_v_{name}':vsc_data['R_v']})
    params_dict.update({f'X_{name}_s':vsc_data['X'],f'R_{name}_s':vsc_data['R']})
    params_dict.update({f'X_{name}_sn':vsc_data['X_n'],f'R_{name}_sn':vsc_data['R_n']})
    params_dict.update({f'X_{name}_ng':vsc_data['X_ng'],f'R_{name}_ng':vsc_data['R_ng']})
    
    params_dict.update({f'S_n_{name}':vsc_data['S_n'],f'U_n_{name}':vsc_data['U_n']})
    params_dict.update({f'T_e_{name}':vsc_data['T_e']})
    params_dict.update({f'T_c_{name}':vsc_data['T_c']})
    params_dict.update({f'Droop_{name}':vsc_data['Droop']})
    params_dict.update({f'K_p_{name}':vsc_data['K_p']})
    params_dict.update({f'T_p_{name}':vsc_data['T_p']})
    params_dict.update({f'T_w_{name}':vsc_data['T_w']})
    params_dict.update({f'K_sec_{name}':vsc_data['K_sec']})
    params_dict.update({f'K_delta_{name}':vsc_data['K_delta']})
    
    
    
def test():
    import numpy as np
    import sympy as sym
    import json
    from pydae.urisi.urisi_builder import urisi
    import pydae.build_cffi as db

    grid = urisi('ac3ph4wgfpidq3_ib.hjson')
    grid.uz_jacs = True
    grid.construct('temp')
    grid.compile('temp')

    import temp

    model = temp.model()
    # model.ini({'p_load_A2_a':20e3,'q_load_A2_a':-10e3,
    #            'p_load_A2_b':20e3,'q_load_A2_b':-10e3,
    #            'p_load_A2_c':20e3,'q_load_A2_c':-10e3,},'xy_0.json')
    
    model.ini({'p_c_A1':10e3},'xy_0.json')
    
    model.report_u()
    model.report_x()
    model.report_y()
    model.report_z()


if __name__ == '__main__':

    #development()
    test()



"""
Created on Thu August 10 23:52:55 2022

@author: jmmauricio
"""

import numpy as np
import sympy as sym

def milano4ord(grid,name,bus_name,data_dict):
    r"""
    Synchronous machine model of order 4 from Federico Milano book.

    """

    sin = sym.sin
    cos = sym.cos  

    # inputs
    V = sym.Symbol(f"V_{bus_name}", real=True)
    theta = sym.Symbol(f"theta_{bus_name}", real=True)
    p_m = sym.Symbol(f"p_m_{name}", real=True)
    v_f = sym.Symbol(f"v_f_{name}", real=True)  
    omega_coi = sym.Symbol("omega_coi", real=True)   
        
    # dynamic states
    delta = sym.Symbol(f"delta_{name}", real=True)
    omega = sym.Symbol(f"omega_{name}", real=True)
    e1q = sym.Symbol(f"e1q_{name}", real=True)
    e1d = sym.Symbol(f"e1d_{name}", real=True)

    # algebraic states
    i_d = sym.Symbol(f"i_d_{name}", real=True)
    i_q = sym.Symbol(f"i_q_{name}", real=True)            
    p_g = sym.Symbol(f"p_g_{name}", real=True)
    q_g = sym.Symbol(f"q_g_{name}", real=True)

    # parameters
    S_n = sym.Symbol(f"S_n_{name}", real=True)
    Omega_b = sym.Symbol(f"Omega_b_{name}", real=True)            
    H = sym.Symbol(f"H_{name}", real=True)
    T1d0 = sym.Symbol(f"T1d0_{name}", real=True)
    T1q0 = sym.Symbol(f"T1q0_{name}", real=True)
    X_d = sym.Symbol(f"X_d_{name}", real=True)
    X_q = sym.Symbol(f"X_q_{name}", real=True)
    X1d = sym.Symbol(f"X1d_{name}", real=True)
    X1q = sym.Symbol(f"X1q_{name}", real=True)
    D = sym.Symbol(f"D_{name}", real=True)
    R_a = sym.Symbol(f"R_a_{name}", real=True)
    K_delta = sym.Symbol(f"K_delta_{name}", real=True)
    K_sat = sym.Symbol(f"K_sat_{name}", real=True)
    params_list = ['S_n','H','T1d0','T1q0','X_d','X_q','X1d','X1q','D','R_a','K_delta','K_sec']
    
    # auxiliar
    v_d = V*sin(delta - theta) 
    v_q = V*cos(delta - theta) 
    p_e = i_d*(v_d + R_a*i_d) + i_q*(v_q + R_a*i_q)     
    omega_s = omega_coi
                
    # dynamic equations            
    ddelta = Omega_b*(omega - omega_s) - K_delta*delta
    domega = 1/(2*H)*(p_m - p_e - D*(omega - omega_s))
    de1q = 1/T1d0*(-e1q*K_sat - (X_d - X1d)*i_d + v_f)
    de1d = 1/T1q0*(-e1d + (X_q - X1q)*i_q)

    # algebraic equations   
    g_i_d  = v_q + R_a*i_q + X1d*i_d - e1q
    g_i_q  = v_d + R_a*i_d - X1q*i_q - e1d
    g_p_g  = i_d*v_d + i_q*v_q - p_g  
    g_q_g  = i_d*v_q - i_q*v_d - q_g 
    
    # dae 
    f_syn = [ddelta,domega,de1q,de1d]
    x_syn = [ delta, omega, e1q, e1d]
    g_syn = [g_i_d,g_i_q,g_p_g,g_q_g]
    y_syn = [  i_d,  i_q,  p_g,  q_g]
    
    grid.H_total += H
    grid.omega_coi_numerator += omega*H*S_n
    grid.omega_coi_denominator += H*S_n

    grid.dae['f'] += f_syn
    grid.dae['x'] += x_syn
    grid.dae['g'] += g_syn
    grid.dae['y_ini'] += y_syn  
    grid.dae['y_run'] += y_syn  
    
    if 'v_f' in data_dict:
        grid.dae['u_ini_dict'].update({f'{v_f}':{data_dict['v_f']}})
        grid.dae['u_run_dict'].update({f'{v_f}':{data_dict['v_f']}})
    else:
        grid.dae['u_ini_dict'].update({f'{v_f}':1.0})
        grid.dae['u_run_dict'].update({f'{v_f}':1.0})

    if 'p_m' in data_dict:
        grid.dae['u_ini_dict'].update({f'{p_m}':{data_dict['p_m']}})
        grid.dae['u_run_dict'].update({f'{p_m}':{data_dict['p_m']}})
    else:
        grid.dae['u_ini_dict'].update({f'{p_m}':1.0})
        grid.dae['u_run_dict'].update({f'{p_m}':1.0})

    grid.dae['xy_0_dict'].update({str(omega):1.0})
    grid.dae['xy_0_dict'].update({str(e1q):1.0})
    grid.dae['xy_0_dict'].update({str(i_q):0.5})

    
    # outputs
    grid.dae['h_dict'].update({f"p_e_{name}":p_e})
    grid.dae['h_dict'].update({f"v_f_{name}":v_f})
    grid.dae['h_dict'].update({f"p_m_{name}":p_m})
    
    if 'F_n' in data_dict:
        grid.dae['params_dict'].update({f"Omega_b_{name}":2*np.pi*data_dict['F_n']})
    else:
        grid.dae['params_dict'].update({f"Omega_b_{name}":data_dict['Omega_b']})

    for item in params_list:       
        grid.dae['params_dict'].update({f"{item}_{name}":data_dict[item]})
    
    grid.dae['params_dict'].update({f"K_sat_{name}":1.0})

    # if 'avr' in syn_data:
    #     add_avr(grid.dae,syn_data)
    #     grid.dae['u_ini_dict'].pop(str(v_f))
    #     grid.dae['u_run_dict'].pop(str(v_f))
    #     grid.dae['xy_0_dict'].update({str(v_f):1.5})

    # if 'gov' in syn_data:
    #     add_gov(grid.dae,syn_data)  
    #     grid.dae['u_ini_dict'].pop(str(p_m))
    #     grid.dae['u_run_dict'].pop(str(p_m))
    #     grid.dae['xy_0_dict'].update({str(p_m):0.5})

    # if 'pss' in syn_data:
    #     add_pss(grid.dae,syn_data)  

    p_W   = p_g * S_n
    q_var = q_g * S_n

    return p_W,q_var


def test():
    import numpy as np
    import sympy as sym
    import hjson
    from pydae.bmapu.bmapu_builder import bmapu
    import pydae.build_cffi as db
    import pytest

    grid = bmapu('milano4ord.hjson')
    grid.checker()
    grid.uz_jacs = True
    grid.build('temp')

    import temp

    model = temp.model()

    v_ref_1 = 1.05
    model.ini({'p_m_1':0.5,'v_ref_1':v_ref_1},'xy_0.json')

    # assert model.get_value('V_1') == pytest.approx(v_ref_1, rel=0.001)

    model.run(1.0,{})
    model.run(1.1,{'b_shunt_1':-500})
    #model.run(2.0,{'b_shunt_1':0})

    model.post()
    # # assert model.get_value('q_A2') == pytest.approx(-q_ref, rel=0.05)

    # model.ini({'p_m_1':0.5,'v_ref_1':1.0},'xy_0.json')
    # model.run(1.0,{})
    # model.run(15.0,{'v_ref_1':1.05})
    # model.post()

    import matplotlib.pyplot as plt

    fig,axes = plt.subplots()
    axes.plot(model.Time,model.get_values('V_1'))
    fig.savefig('milano4ord_fault.svg')


if __name__ == '__main__':

    #development()
    test()


# -*- coding: utf-8 -*-
"""
Created on Thu August 10 23:52:55 2022

@author: jmmauricio
"""


import sympy as sym


def ntsieeeg1(dae,syn_data,name,bus_name):
    '''

    .. table:: Constants
        :widths: auto

        ================== =========== ============================================= =========== 
        Variable           Code        Description                                   Units
        ================== =========== ============================================= ===========
        :math:`T_A`        ``T_a``     Time Constant                                 s
        ================== =========== ============================================= ===========

    Example:
    
    ``"gov":{"type":"ntsst4","K":20,"K_1":0.3,"K_3":0.3,"K_5":0.4,"K_7":0,"T_1":0,"T_2":0.0,"T_3":0.1,"T_4":0.3,"T_5":7, "T_6":0.6, "T_7":0,
 "K_2":0,"K_4":0,"K_6":0,"K_8":0,"U_0":0.5, "U_c":-0.5,"p_c":0.1}
    '''

    data = syn_data['gov']
    
    omega = sym.Symbol(f"omega_{name}", real=True)   

    p_c = sym.Symbol(f"p_c_{name}", real=True)  
    p_agc = sym.Symbol(f"p_agc", real=True)  

    x_3 = sym.Symbol(f"x_3_gov_{name}", real=True);
    x_4 = sym.Symbol(f"x_4_gov_{name}", real=True);
    x_5 = sym.Symbol(f"x_5_gov_{name}", real=True);
    x_6 = sym.Symbol(f"x_6_gov_{name}", real=True);
    p_m = sym.Symbol(f"p_m_{name}", real=True)  

    K    = sym.Symbol(f"K_gov_{name}", real=True)
    K_1  = sym.Symbol(f"K_1_gov_{name}", real=True)
    K_3  = sym.Symbol(f"K_3_gov_{name}", real=True)
    K_5  = sym.Symbol(f"K_5_gov_{name}", real=True)
    K_7  = sym.Symbol(f"K_7_gov_{name}", real=True)
    T_1  = sym.Symbol(f"T_1_gov_{name}", real=True)
    T_2  = sym.Symbol(f"T_2_gov_{name}", real=True)
    T_3  = sym.Symbol(f"T_3_gov_{name}", real=True)
    T_4  = sym.Symbol(f"T_4_gov_{name}", real=True)
    T_5  = sym.Symbol(f"T_5_gov_{name}", real=True)
    T_6  = sym.Symbol(f"T_6_gov_{name}", real=True)
    T_7  = sym.Symbol(f"T_7_gov_{name}", real=True)
    K_2  = sym.Symbol(f"K_2_gov_{name}", real=True)
    K_4  = sym.Symbol(f"K_4_gov_{name}", real=True)
    K_6  = sym.Symbol(f"K_6_gov_{name}", real=True)
    K_8  = sym.Symbol(f"K_8_gov_{name}", real=True)
    U_0  = sym.Symbol(f"U_0_gov_{name}", real=True)
    U_c  = sym.Symbol(f"U_c_gov_{name}", real=True) 
    P_min  = sym.Symbol(f"P_min_gov_{name}", real=True)
    P_max  = sym.Symbol(f"P_max_gov_{name}", real=True) 
    K_awu  = sym.Symbol(f"K_awu_gov_{name}", real=True) 
    K_sec = sym.Symbol(f"K_sec_{name}", real=True)    


    Domega = 1.0 - omega

    y_g_nosat = x_3
    y_g = sym.Piecewise((P_max,y_g_nosat > P_max),(P_min,y_g_nosat < P_min),(y_g_nosat,True))
    # y_g = y_g_nosat

    u_3 = K*Domega + p_c - y_g + K_sec*p_agc
    u_g = sym.Piecewise((U_c,u_3/T_3<U_c),(U_0,u_3/T_3>U_0),(u_3/T_3,True))
    u_awu = K_awu*(y_g - y_g_nosat)

    dx_3 = u_g + u_awu
    dx_4 = 1/T_4*(y_g - x_4)
    dx_5 = 1/T_5*(x_4 - x_5)
    dx_6 = 1/T_6*(x_5 - x_6)

    g_p_m = -p_m + K_1*x_4 + K_3*x_5 + K_5*x_6

    dae['f'] += [dx_3,dx_4,dx_5,dx_6]
    dae['x'] += [ x_3, x_4, x_5, x_6]
    dae['g'] += [g_p_m]
    dae['y_ini'] += [p_m] 
    dae['y_run'] += [p_m]  

    dae['params_dict'].update({str(K):data['K']})
    dae['params_dict'].update({str(K_1):data['K_1']})
    dae['params_dict'].update({str(K_3):data['K_3']})
    dae['params_dict'].update({str(K_5):data['K_5']})
    dae['params_dict'].update({str(K_7):data['K_7']})
    dae['params_dict'].update({str(T_1):data['T_1']})
    dae['params_dict'].update({str(T_2):data['T_2']})
    dae['params_dict'].update({str(T_3):data['T_3']})
    dae['params_dict'].update({str(T_4):data['T_4']})
    dae['params_dict'].update({str(T_5):data['T_5']})
    dae['params_dict'].update({str(T_6):data['T_6']})
    dae['params_dict'].update({str(T_7):data['T_7']})
    dae['params_dict'].update({str(K_2):data['K_2']})
    dae['params_dict'].update({str(K_4):data['K_4']})
    dae['params_dict'].update({str(K_6):data['K_6']})
    dae['params_dict'].update({str(K_8):data['K_8']})
    dae['params_dict'].update({str(U_0):data['U_0']})
    dae['params_dict'].update({str(U_c):data['U_c']})
    dae['params_dict'].update({str(P_max):data['P_max']})
    dae['params_dict'].update({str(P_min):data['P_min']})
    dae['params_dict'].update({str(K_awu):1000.0}) 

    p_c_ini = data['p_c']

    dae['u_ini_dict'].update({str(p_c):p_c_ini})
    dae['u_run_dict'].update({str(p_c):p_c_ini})

    dae['xy_0_dict'].update({str(x_3):p_c_ini})
    dae['xy_0_dict'].update({str(x_4):p_c_ini})
    dae['xy_0_dict'].update({str(x_5):p_c_ini})
    dae['xy_0_dict'].update({str(x_6):p_c_ini})
    dae['xy_0_dict'].update({str(p_m):p_c_ini})    


def test():
    import numpy as np
    import sympy as sym
    import hjson
    from pydae.bmapu.bmapu_builder import bmapu
    import pydae.build_cffi as db
    import matplotlib.pyplot as plt
    import pytest

    grid = bmapu('ieeeg1.hjson')
    grid.checker()
    grid.uz_jacs = True
    grid.build('temp')

    import temp

    model = temp.model()

    model.ini({'P_2':-80e6},'xy_0.json')
    model.ini({})
    model.report_x()
    model.report_y()
    model.report_z()

    model.Dt = 0.01

    model.run(10.0,{})
    model.run(20.0,{'P_2':-100e6})
    model.post();

    fig,axes = plt.subplots()
    axes.plot(model.Time,model.get_values('omega_coi'))
    fig.savefig('ieeeg1.svg')




if __name__ == '__main__':

    #development()
    test()









 



import numpy as np
import sympy as sym

def ctrl_3ph_4w_droop(grid,vsc_data,ctrl_data,name,bus_name):

    bus_ac = vsc_data['bus_ac']
    bus_dc = vsc_data['bus_dc']

    buses_names = [bus['name'] for bus in grid.data['buses']]
    idx = buses_names.index(bus_ac)
    U_ac_b = grid.data['buses'][idx]['U_kV']*1e3
    idx = buses_names.index(bus_dc)
    V_dc_b = grid.data['buses'][idx]['U_kV']*1e3


    V_phn = []
    n2a = {0:'a',1:'b',2:'c'}
    # phase top neutral voltages:
    V_n_r = sym.Symbol(f'V_{bus_ac}_{3}_r', real = True)
    V_n_i = sym.Symbol(f'V_{bus_ac}_{3}_i', real = True)

    # phase-neutral voltage module
    for ph in [0,1,2]:
        V_ph_r = sym.Symbol(f'V_{bus_ac}_{ph}_r', real = True)
        V_ph_i = sym.Symbol(f'V_{bus_ac}_{ph}_i', real = True)
        z_name = f'V_{bus_ac}_{n2a[ph]}n'
        z_value = ((V_ph_r-V_n_r)**2 + (V_ph_i-V_n_i)**2)**0.5
        V_phn += [z_value]

    # phase-neutral voltage module
    V_n_r = sym.Symbol(f'V_{bus_dc}_{1}_r', real = True)
    V_n_i = sym.Symbol(f'V_{bus_dc}_{1}_i', real = True)
    V_ph_r = sym.Symbol(f'V_{bus_dc}_{0}_r', real = True)
    V_ph_i = sym.Symbol(f'V_{bus_dc}_{0}_i', real = True)
    v_dc = ((V_ph_r-V_n_r)**2 + (V_ph_i-V_n_i)**2)**0.5

    K_acdc = sym.Symbol(f'K_acdc_{bus_ac}', real=True)
    p_vsc_a,p_vsc_b,p_vsc_c = sym.symbols(f'p_vsc_a_{bus_ac},p_vsc_b_{bus_ac},p_vsc_c_{bus_ac}',real=True)

    V_ac_b = U_ac_b/np.sqrt(3)

    v_dc_pu = v_dc/800.0
    p_ac_a = K_acdc*(v_dc_pu - V_phn[0]/V_ac_b)
    p_ac_b = K_acdc*(v_dc_pu - V_phn[1]/V_ac_b)
    p_ac_c = K_acdc*(v_dc_pu - V_phn[2]/V_ac_b)

    grid.dae['g'] += [p_ac_a - p_vsc_a]
    grid.dae['g'] += [p_ac_b - p_vsc_b]
    grid.dae['g'] += [p_ac_c - p_vsc_c]

    grid.dae['y_ini'] += [p_vsc_a]
    grid.dae['y_ini'] += [p_vsc_b]
    grid.dae['y_ini'] += [p_vsc_c]

    grid.dae['y_run'] += [p_vsc_a]
    grid.dae['y_run'] += [p_vsc_b]
    grid.dae['y_run'] += [p_vsc_c]

    grid.dae['params_dict'].update({f'K_acdc_{bus_ac}':0.0})

    grid.dae['u_ini_dict'].pop(f'p_vsc_a_{bus_ac}')
    grid.dae['u_ini_dict'].pop(f'p_vsc_b_{bus_ac}')
    grid.dae['u_ini_dict'].pop(f'p_vsc_c_{bus_ac}')
    grid.dae['u_run_dict'].pop(f'p_vsc_a_{bus_ac}')
    grid.dae['u_run_dict'].pop(f'p_vsc_b_{bus_ac}')
    grid.dae['u_run_dict'].pop(f'p_vsc_c_{bus_ac}')

    grid.dae['h_dict'].update({f'v_ac_a_pu_{bus_ac}':V_phn[0]/V_ac_b})
    grid.dae['h_dict'].update({f'v_ac_b_pu_{bus_ac}':V_phn[1]/V_ac_b})
    grid.dae['h_dict'].update({f'v_ac_c_pu_{bus_ac}':V_phn[2]/V_ac_b})
    grid.dae['h_dict'].update({f'v_dc_pu_{bus_dc}':v_dc_pu})



# -*- coding: utf-8 -*-
"""
Created on Thu August 10 23:52:55 2022

@author: jmmauricio
"""

import numpy as np
import sympy as sym


def pss_pss2(dae,syn_data,name,bus_name):
    '''
    PSS as IEEE PSS2
    'K_s3':1.0,'T_wo1':2.0,'T_wo2':2.0,'T_9': 0.1,'K_s1':17.069,'T_1': 0.28,'T_2':0.04,'T_3':0.28,'T_4':0.12,'T_wo3':2.0,'K_s2': 0.158,'T_7':2.0

    '''
    pss_data = syn_data['pss']
    

    omega = sym.Symbol(f"omega_{name}", real=True)   
    p_g   = sym.Symbol(f"p_g_{name}", real=True) 

    # states
    x_wo1 = sym.Symbol(f"x_wo1_{name}", real=True)
    x_wo2 = sym.Symbol(f"x_wo2_{name}", real=True)
    x_wo3 = sym.Symbol(f"x_wo3_{name}", real=True)
    x_lpf7= sym.Symbol(f"x_lpf7_{name}", real=True)
    x_9_1 = sym.Symbol(f"x_9_1_{name}", real=True)
    x_9_2 = sym.Symbol(f"x_9_2_{name}", real=True)
    x_9_3 = sym.Symbol(f"x_9_3_{name}", real=True)
    x_9_4 = sym.Symbol(f"x_9_4_{name}", real=True)
    x_9_5 = sym.Symbol(f"x_9_5_{name}", real=True)
    x_ll1 = sym.Symbol(f"x_ll1_{name}", real=True)
    x_ll3 = sym.Symbol(f"x_ll3_{name}", real=True)

 
    v_pss = sym.Symbol(f"v_pss_{name}", real=True) 
    
    T_1   = sym.Symbol(f"T_pss_1_{name}", real=True)    
    T_2   = sym.Symbol(f"T_pss_2_{name}", real=True)    
    T_3   = sym.Symbol(f"T_pss_3_{name}", real=True)    
    T_4   = sym.Symbol(f"T_pss_4_{name}", real=True) 
    T_7   = sym.Symbol(f"T_pss_7_{name}", real=True)   
    T_9   = sym.Symbol(f"T_pss_9_{name}", real=True) 

    T_wo1 = sym.Symbol(f"T_wo1_{name}", real=True)  
    T_wo2 = sym.Symbol(f"T_wo2_{name}", real=True)  
    T_wo3 = sym.Symbol(f"T_wo3_{name}", real=True) 

    K_s1  = sym.Symbol(f"K_s1_{name}", real=True)   
    K_s2  = sym.Symbol(f"K_s2_{name}", real=True)   
    K_s3  = sym.Symbol(f"K_s3_{name}", real=True) 

    u_1 = omega
    u_2 = p_g

    y_9 = x_9_5 
    z_wo1 = u_1 - x_wo1 
    z_wo2 = z_wo1 - x_wo2 

    y_3 = x_lpf7 
    u_9 = K_s3*y_3 + z_wo2 
    u_ll = K_s1*(y_9 - y_3) 
    z_wo3 = u_2 - x_wo3 

    dx_wo1 =   (u_1 - x_wo1)/T_wo1
    dx_wo2 =   (z_wo1 - x_wo2)/T_wo2 
    dx_wo3 =   (u_2 - x_wo3)/T_wo3 
    dx_lpf7 =   (K_s2*z_wo3 - x_lpf7)/T_7 
    dx_9_1 = 1/T_9*(u_9 - x_9_1) 
    dx_9_2 = 1/T_9*(x_9_1 - x_9_2)
    dx_9_3 = 1/T_9*(x_9_2 - x_9_3)
    dx_9_4 = 1/T_9*(x_9_3 - x_9_4)
    dx_9_5 = 1/T_9*(x_9_4 - x_9_5)
    dx_ll1 =( u_ll - x_ll1)/T_2;  
    z_ll1 = ( u_ll - x_ll1)*T_1/T_2 + x_ll1 
    dx_ll3 =(z_ll1 - x_ll3)/T_4;   
    z_ll3 = (z_ll1 - x_ll3)*T_3/T_4 + x_ll3

    g_v_pss = -v_pss + z_ll3  
    
    dae['f'] += [dx_wo1,dx_wo2,dx_wo3,dx_lpf7,dx_9_1,dx_9_2,dx_9_3,dx_9_4,dx_9_5,dx_ll1,dx_ll3]
    dae['x'] += [ x_wo1, x_wo2, x_wo3, x_lpf7, x_9_1, x_9_2, x_9_3, x_9_4, x_9_5, x_ll1, x_ll3]
    dae['g'] += [g_v_pss]
    dae['y_ini'] += [  v_pss]  
    dae['y_run'] += [  v_pss] 

    dae['params_dict'].update({str(T_1):pss_data['T_1']})
    dae['params_dict'].update({str(T_2):pss_data['T_2']})
    dae['params_dict'].update({str(T_3):pss_data['T_3']})
    dae['params_dict'].update({str(T_4):pss_data['T_4']})
    dae['params_dict'].update({str(T_7):pss_data['T_7']})
    dae['params_dict'].update({str(T_9):pss_data['T_9']})
    dae['params_dict'].update({str(T_wo1):pss_data['T_wo1']})
    dae['params_dict'].update({str(T_wo2):pss_data['T_wo2']})
    dae['params_dict'].update({str(T_wo3):pss_data['T_wo3']})
    dae['params_dict'].update({str(K_s1):pss_data['K_s1']})
    dae['params_dict'].update({str(K_s2):pss_data['K_s2']})
    dae['params_dict'].update({str(K_s3):pss_data['K_s3']})


# -*- coding: utf-8 -*-
"""
Created on Thu August 10 23:52:55 2022

@author: jmmauricio
"""

import numpy as np
import sympy as sym
from pydae.bmapu.psss.pss_kundur import pss_kundur
from pydae.bmapu.psss.pss_pss2 import pss_pss2
from pydae.bmapu.psss.pss_kundur_2 import pss_kundur_2

def add_pss(dae,syn_data,name,bus_name):
    
    if syn_data['pss']['type'] == 'pss_kundur':
        pss_kundur(dae,syn_data,name,bus_name)
    elif syn_data['pss']['type'] == 'pss2':
        pss_pss2(dae,syn_data,name,bus_name)
    elif syn_data['pss']['type'] == 'pss_kundur_2':
        pss_kundur_2(dae,syn_data,name,bus_name)
    else:
        print(f"PSS type {syn_data['pss']['type']} not found.")



# -*- coding: utf-8 -*-
"""
Created on Thu August 10 23:52:55 2022

@author: jmmauricio
"""

import numpy as np
import sympy as sym

def add_model(grid,name,bus_name,data_dict):
    '''


    
    '''

    pass



@numba.njit("(float64[:],float64[:],float64[:],float64[:],float64[:],float64)")
def sp_Fu_run_eval(jac,x,y,u,p,Dt):   
    '''
    Computes the dense full initialization jacobian:
    
    jac_ini = [[Fx_ini, Fy_ini],
               [Gx_ini, Gy_ini]]
                
    for the given x,y,u,p vectors and Dt time increment.
    
    Parameters
    ----------
    de_jac_ini : (N, N) array_like
                  Input data.
    x : (N_x,) array_like
        Vector with dynamical states.
    y : (N_y,) array_like
        Vector with algebraic states (ini problem).
    u : (N_u,) array_like
        Vector with inputs (ini problem). 
    p : (N_p,) array_like
        Vector with parameters. 
        
    with N = N_x+N_y
 
    Returns
    -------
    
    de_jac_ini : (N, N) array_like
                  Updated matrix.    
    
    '''
    
    jac_ptr=ffi.from_buffer(np.ascontiguousarray(jac))
    x_c_ptr=ffi.from_buffer(np.ascontiguousarray(x))
    y_c_ptr=ffi.from_buffer(np.ascontiguousarray(y))
    u_c_ptr=ffi.from_buffer(np.ascontiguousarray(u))
    p_c_ptr=ffi.from_buffer(np.ascontiguousarray(p))

    sp_Fu_run_up_eval( jac_ptr,x_c_ptr,y_c_ptr,u_c_ptr,p_c_ptr,Dt)
    sp_Fu_run_xy_eval( jac_ptr,x_c_ptr,y_c_ptr,u_c_ptr,p_c_ptr,Dt)
    
    #return jac

@numba.njit("(float64[:],float64[:],float64[:],float64[:],float64[:],float64)")
def sp_Gu_run_eval(jac,x,y,u,p,Dt):   
    '''
    Computes the dense full initialization jacobian:
    
    jac_ini = [[Fx_ini, Fy_ini],
               [Gx_ini, Gy_ini]]
                
    for the given x,y,u,p vectors and Dt time increment.
    
    Parameters
    ----------
    de_jac_ini : (N, N) array_like
                  Input data.
    x : (N_x,) array_like
        Vector with dynamical states.
    y : (N_y,) array_like
        Vector with algebraic states (ini problem).
    u : (N_u,) array_like
        Vector with inputs (ini problem). 
    p : (N_p,) array_like
        Vector with parameters. 
        
    with N = N_x+N_y
 
    Returns
    -------
    
    de_jac_ini : (N, N) array_like
                  Updated matrix.    
    
    '''
    
    jac_ptr=ffi.from_buffer(np.ascontiguousarray(jac))
    x_c_ptr=ffi.from_buffer(np.ascontiguousarray(x))
    y_c_ptr=ffi.from_buffer(np.ascontiguousarray(y))
    u_c_ptr=ffi.from_buffer(np.ascontiguousarray(u))
    p_c_ptr=ffi.from_buffer(np.ascontiguousarray(p))

    sp_Gu_run_up_eval( jac_ptr,x_c_ptr,y_c_ptr,u_c_ptr,p_c_ptr,Dt)
    sp_Gu_run_xy_eval( jac_ptr,x_c_ptr,y_c_ptr,u_c_ptr,p_c_ptr,Dt)
    
    #return jac

@numba.njit("(float64[:],float64[:],float64[:],float64[:],float64[:],float64[:],float64[:],float64)")
def sp_H_jacs_run_eval(H_x,H_y,H_u,x,y,u,p,Dt):   
    '''
    Computes the dense full initialization jacobian:
    
    jac_ini = [[Fx_ini, Fy_ini],
               [Gx_ini, Gy_ini]]
                
    for the given x,y,u,p vectors and Dt time increment.
    
    Parameters
    ----------
    de_jac_ini : (N, N) array_like
                  Input data.
    x : (N_x,) array_like
        Vector with dynamical states.
    y : (N_y,) array_like
        Vector with algebraic states (ini problem).
    u : (N_u,) array_like
        Vector with inputs (ini problem). 
    p : (N_p,) array_like
        Vector with parameters. 
        
    with N = N_x+N_y
 
    Returns
    -------
    
    de_jac_ini : (N, N) array_like
                  Updated matrix.    
    
    '''
    
    H_x_ptr=ffi.from_buffer(np.ascontiguousarray(H_x))
    H_y_ptr=ffi.from_buffer(np.ascontiguousarray(H_y))
    H_u_ptr=ffi.from_buffer(np.ascontiguousarray(H_u))

    x_c_ptr=ffi.from_buffer(np.ascontiguousarray(x))
    y_c_ptr=ffi.from_buffer(np.ascontiguousarray(y))
    u_c_ptr=ffi.from_buffer(np.ascontiguousarray(u))
    p_c_ptr=ffi.from_buffer(np.ascontiguousarray(p))

    sp_Hx_run_up_eval( H_x_ptr,x_c_ptr,y_c_ptr,u_c_ptr,p_c_ptr,Dt)
    sp_Hx_run_xy_eval( H_x_ptr,x_c_ptr,y_c_ptr,u_c_ptr,p_c_ptr,Dt)
    sp_Hy_run_up_eval( H_y_ptr,x_c_ptr,y_c_ptr,u_c_ptr,p_c_ptr,Dt)
    sp_Hy_run_xy_eval( H_y_ptr,x_c_ptr,y_c_ptr,u_c_ptr,p_c_ptr,Dt)
    sp_Hu_run_up_eval( H_u_ptr,x_c_ptr,y_c_ptr,u_c_ptr,p_c_ptr,Dt)
    sp_Hu_run_xy_eval( H_u_ptr,x_c_ptr,y_c_ptr,u_c_ptr,p_c_ptr,Dt)


# -*- coding: utf-8 -*-
"""
Created on Thu August 10 23:52:55 2022

@author: jmmauricio
"""

import numpy as np
import sympy as sym
from pydae.urisi.loads.load_ac import load_ac
from pydae.urisi.loads.load_ac_3w import load_ac_3w
from pydae.urisi.loads.load_dc import load_dc


def add_loads(grid):

    buses = grid.data['buses']
    buses_list = [bus['name'] for bus in buses]

    self = grid


    # loads
    for load in self.data['loads']:
        if load['type'] == '3P+N' and  load["model"] == 'ZIP':
            load_ac(grid,load)
        if load['type'] == '3P' and  load["model"] == 'ZIP':
            load_ac_3w(grid,load)
        if load['type'] == 'DC' and  load["model"] == 'ZIP':
            load_dc(grid,load)


# -*- coding: utf-8 -*-
"""
Created on Thu August 10 23:52:55 2022

@author: jmmauricio
"""

import numpy as np
import sympy as sym



def load_ac_3w(grid,data):

    buses = grid.data['buses']
    buses_list = [bus['name'] for bus in buses]

    self = grid

    name = data['bus']
    v_sa_r,v_sb_r,v_sc_r = sym.symbols(f'V_{name}_0_r,V_{name}_1_r,V_{name}_2_r', real=True)
    v_sa_i,v_sb_i,v_sc_i = sym.symbols(f'V_{name}_0_i,V_{name}_1_i,V_{name}_2_i', real=True)
    K_abc,V_th = sym.symbols(f'K_abc_{name},V_th_{name}',real=True)
    # i_a = I_node_sym_list[nodes_list.index(f'{bus_name}.1')]
    # i_b = I_node_sym_list[nodes_list.index(f'{bus_name}.2')]
    # i_c = I_node_sym_list[nodes_list.index(f'{bus_name}.3')]
    # i_n = I_node_sym_list[nodes_list.index(f'{bus_name}.4')]

    i_a_r,i_a_i = sym.symbols(f'i_load_{name}_a_r,i_load_{name}_a_i', real=True)
    i_b_r,i_b_i = sym.symbols(f'i_load_{name}_b_r,i_load_{name}_b_i', real=True)
    i_c_r,i_c_i = sym.symbols(f'i_load_{name}_c_r,i_load_{name}_c_i', real=True)
    
    i_a = i_a_r + sym.I*i_a_i
    i_b = i_b_r + sym.I*i_b_i
    i_c = i_c_r + sym.I*i_c_i
    
    v_a = v_sa_r +  sym.I*v_sa_i
    v_b = v_sb_r +  sym.I*v_sb_i
    v_c = v_sc_r +  sym.I*v_sc_i

    s_a = v_a*sym.conjugate(i_a)
    s_b = v_b*sym.conjugate(i_b)
    s_c = v_c*sym.conjugate(i_c)
    #s = s_a + s_b + s_c

    p,q = sym.symbols(f'p_load_{name},q_load_{name}', real=True)
    
    self.dae['g'] += [p/3 + sym.re(s_a)]
    self.dae['g'] += [p/3 + sym.re(s_b)]
    self.dae['g'] += [q/3 + sym.im(s_a)]
    self.dae['g'] += [q/3 + sym.im(s_b)]

    self.dae['g'] += [sym.re(i_a+i_b+i_c)]
    self.dae['g'] += [sym.im(i_a+i_b+i_c)]

    self.dae['y_ini'] += [i_a_r]
    self.dae['y_ini'] += [i_a_i]
    self.dae['y_ini'] += [i_b_r]
    self.dae['y_ini'] += [i_b_i]
    self.dae['y_ini'] += [i_c_r]
    self.dae['y_ini'] += [i_c_i]

    self.dae['y_run'] += [i_a_r]
    self.dae['y_run'] += [i_a_i]
    self.dae['y_run'] += [i_b_r]
    self.dae['y_run'] += [i_b_i]
    self.dae['y_run'] += [i_c_r]
    self.dae['y_run'] += [i_c_i]


    for ph in ['a','b','c']:
        i_s_r,i_s_i = sym.symbols(f'i_load_{name}_{ph}_r,i_load_{name}_{ph}_i', real=True)
        idx_r,idx_i = self.node2idx(name,ph)
        self.dae['g'] [idx_r] += -i_s_r
        self.dae['g'] [idx_i] += -i_s_i


    p_load_N,q_load_N = 0.0,0.0
    if 'kVA' in data:
        S_N = data['kVA']*1000
        pf_N = data['pf']
        p_load_N = S_N*np.abs(pf_N)
        q_load_N = np.sqrt(S_N**2 - p_load_N**2)*np.sign(pf_N)
    if 'kW' in data:
        p_load_N =  data['kW']*1000
        q_load_N =  data['kvar']*1000
        
    self.dae['u_ini_dict'].update({f'p_load_{name}':p_load_N})
    self.dae['u_ini_dict'].update({f'q_load_{name}':q_load_N})
    self.dae['u_run_dict'].update({f'p_load_{name}':p_load_N})
    self.dae['u_run_dict'].update({f'q_load_{name}':q_load_N})





# -*- coding: utf-8 -*-
"""
Created on Thu August 10 23:52:55 2022

@author: jmmauricio
"""


import sympy as sym


def ntsst1(dae,syn_data,name,bus_name):
    '''

    .. table:: Constants
        :widths: auto

    Example:
    
    ``"avr":{"type":"ntsst1","K_a":200.0,"T_c":1.0,"T_b":10.0,"v_ref":1.0},``

    '''


    avr_data = syn_data['avr']
    remote_bus_name = bus_name
    if 'bus' in avr_data:
        remote_bus_name = avr_data['bus']

    v_t = sym.Symbol(f"V_{remote_bus_name}", real=True)  
    v_r = sym.Symbol(f"v_r_{name}", real=True)   
    v_c = sym.Symbol(f"v_c_{remote_bus_name}", real=True)  
    x_cb  = sym.Symbol(f"x_cb_{name}", real=True)
    xi_v  = sym.Symbol(f"xi_v_{name}", real=True)
    v_f = sym.Symbol(f"v_f_{name}", real=True)  
    T_r = sym.Symbol(f"T_r_{name}", real=True) 
    T_b = sym.Symbol(f"T_b_{name}", real=True) 
    T_c = sym.Symbol(f"T_c_{name}", real=True) 

    K_a = sym.Symbol(f"K_a_{name}", real=True)
    K_ai = sym.Symbol(f"K_ai_{name}", real=True)

    V_f_min = sym.Symbol(f"V_f_min_{name}", real=True)
    V_f_max = sym.Symbol(f"V_f_max_{name}", real=True)

    v_ref = sym.Symbol(f"v_ref_{name}", real=True) 
    v_pss = sym.Symbol(f"v_pss_{name}", real=True) 

    k_sat = sym.Symbol(f"k_sat_{name}", real=True) 

    v_s = v_pss # v_oel and v_uel are not considered
    v_ini = K_ai*xi_v

    v_c = v_t # no droop is considered
    v_1 = v_ref - v_c + v_s + v_ini # v_ini is added in pydae to force V = v_ref in the initialization
    
    epsilon_v = v_ref - v_c 
    
    dv_r = 1/T_r*(v_c - v_r)
    dx_cb = (v_1 - x_cb)/T_b;  
    z_cb  = (v_1 - x_cb)*T_c/T_b + x_cb 
    dxi_v = epsilon_v  # this integrator is added in pydae to force V = v_ref in the initialization
    
    v_f_nosat = K_a*z_cb
    v_f_sat = sym.Piecewise((V_f_max,v_f_nosat>V_f_max),
                            (V_f_min,v_f_nosat<V_f_min),
                            (v_f_nosat,True))
    #v_f_sat = v_f_nosat
    #g_v_f  =   v_f_sat - v_f 
    v_f_ref = v_f_sat*k_sat + v_f_nosat*(1 - k_sat)
    dv_f = 1/T_r*(v_f_ref - v_f)
    
    dae['f'] += [dv_r,dx_cb,dxi_v,dv_f]
    dae['x'] += [ v_r, x_cb, xi_v, v_f]
    # dae['g'] += [g_v_f]
    # dae['y_ini'] += [v_f] 
    # dae['y_run'] += [v_f]  

    dae['params_dict'].update({str(K_a):avr_data['K_a']})
    dae['params_dict'].update({str(K_ai):1e-6})
    dae['params_dict'].update({str(T_r):avr_data['T_r']})  
    dae['params_dict'].update({str(T_c):avr_data['T_c']})  
    dae['params_dict'].update({str(T_b):avr_data['T_b']})  

    dae['params_dict'].update({str(V_f_max): 100.0})  
    dae['params_dict'].update({str(V_f_min):-100.0})  


    dae['u_ini_dict'].update({str(v_ref):avr_data['v_ref']})
    dae['u_run_dict'].update({str(v_ref):avr_data['v_ref']})

    dae['u_run_dict'].update({str(v_pss):0.0})
    dae['u_ini_dict'].update({str(v_pss):0.0})

    dae['u_ini_dict'].update({str(k_sat):0.0})
    dae['u_run_dict'].update({str(k_sat):1.0})


    dae['xy_0_dict'].update({str(v_f):2.0})
    dae['xy_0_dict'].update({str(v_r):1.0})
    dae['xy_0_dict'].update({str(xi_v):10000.0})
    dae['xy_0_dict'].update({str(x_cb):2.0})

    dae['h_dict'].update({str(v_ref):v_ref})  


def test():
    import numpy as np
    import sympy as sym
    import hjson
    from pydae.bmapu.bmapu_builder import bmapu
    import pydae.build_cffi as db
    import pytest

    grid = bmapu('ntsst1.hjson')
    grid.checker()
    grid.uz_jacs = True
    grid.build('temp')

    import temp

    model = temp.model()

    v_ref_1 = 1.05
    model.ini({},'xy_0.json')
    model.report_x()
    model.report_y()
    model.report_z()

    # assert model.get_value('V_1') == pytest.approx(v_ref_1, rel=0.001)
    # # assert model.get_value('q_A2') == pytest.approx(-q_ref, rel=0.05)

    # model.ini({'p_m_1':0.5,'v_ref_1':1.0},'xy_0.json')
    # model.run(1.0,{})
    # model.run(15.0,{'v_ref_1':1.05})
    # model.post()

    # import matplotlib.pyplot as plt

    # fig,axes = plt.subplots()
    # axes.plot(model.Time,model.get_values('V_1'))
    # fig.savefig('ntsst1_step.svg')


if __name__ == '__main__':

    #development()
    test()



# -*- coding: utf-8 -*-
"""
Created on Thu August 10 23:52:55 2022

@author: jmmauricio
"""


import sympy as sym


def sexs(dae,syn_data,name,bus_name):
    '''

    .. table:: Constants
        :widths: auto

        ================== =========== ============================================= =========== 
        Variable           Code        Description                                   Units
        ================== =========== ============================================= ===========
        :math:`T_A`        ``T_a``     Time Constant                                 s
        :math:`T_B`        ``T_b``     Time Constant                                 s
        :math:`T_E`        ``T_e``     Time Constant                                 s
        :math:`EMAX`       ``E_max``   Limiter                                       pu-m
        :math:`EMIN`       ``E_min``   Limiter                                       pu-m
        ================== =========== ============================================= ===========

    Example:
    
    ``"avr":{'type':'sexs','K_a':100.0,'T_a':0.1,'T_b':1.0,'T_e':0.1,'E_min':-10.0,'E_max':10.0,"v_ref":1.0},``

    '''

    avr_data = syn_data['avr']
    
    v_t = sym.Symbol(f"V_{bus_name}", real=True)   
    v_c = sym.Symbol(f"v_c_{name}", real=True)  
    x_ab  = sym.Symbol(f"x_ab_{name}", real=True)
    x_e  = sym.Symbol(f"x_e_{name}", real=True)
    xi_v  = sym.Symbol(f"xi_v_{name}", real=True)
    v_f = sym.Symbol(f"v_f_{name}", real=True)  
    T_a = sym.Symbol(f"T_a_{name}", real=True) 
    T_b = sym.Symbol(f"T_b_{name}", real=True) 
    T_e = sym.Symbol(f"T_e_{name}", real=True) 

    K_a = sym.Symbol(f"K_a_{name}", real=True)
    K_ai = sym.Symbol(f"K_ai_{name}", real=True)
    E_min = sym.Symbol(f"E_min_{name}", real=True)
    E_max = sym.Symbol(f"E_max_{name}", real=True) 
    
    v_ref = sym.Symbol(f"v_ref_{name}", real=True) 
    v_pss = sym.Symbol(f"v_pss_{name}", real=True) 

    v_s = v_pss # v_oel and v_uel are not considered
    v_ini = K_ai*xi_v

    v_c = v_t # no droop is considered
    v_2 = v_ref - v_c + v_s + v_ini # v_ini is added in pydae to force V = v_ref in the initialization
    
    epsilon_v = v_ref - v_c 
    
    dx_ab = (v_2 - x_ab)/T_b;  
    z_ab  = (v_2 - x_ab)*T_a/T_b + x_ab 
    dx_e  = (K_a*z_ab - x_e)/T_e 
    dxi_v = epsilon_v  # this integrator is added in pydae to force V = v_ref in the initialization

    efd_nosat = x_e + 1.0
    efd = sym.Piecewise((E_min, efd_nosat<E_min),(E_max,efd_nosat>E_max),(efd_nosat,True))
    g_v_f  =   efd - v_f 
    
    dae['f'] += [dx_ab,dx_e,dxi_v]
    dae['x'] += [ x_ab, x_e, xi_v]
    dae['g'] += [g_v_f]
    dae['y_ini'] += [v_f] 
    dae['y_run'] += [v_f]  

    dae['params_dict'].update({str(K_a):avr_data['K_a']})
    dae['params_dict'].update({str(K_ai):1e-6})
    dae['params_dict'].update({str(T_a):avr_data['T_a']})  
    dae['params_dict'].update({str(T_b):avr_data['T_b']})  
    dae['params_dict'].update({str(T_e):avr_data['T_e']})  
    dae['params_dict'].update({str(E_min):avr_data['E_min']})  
    dae['params_dict'].update({str(E_max):avr_data['E_max']})  

    dae['u_ini_dict'].update({str(v_ref):avr_data['v_ref']})
    dae['u_run_dict'].update({str(v_ref):avr_data['v_ref']})

    dae['u_run_dict'].update({str(v_pss):0.0})
    dae['u_ini_dict'].update({str(v_pss):0.0})

    dae['h_dict'].update({str(v_pss):v_pss})
    dae['h_dict'].update({str(v_ref):v_ref})

    dae['xy_0_dict'].update({str(xi_v):0.0})
    dae['xy_0_dict'].update({str(x_e):0.0})
    dae['xy_0_dict'].update({str(v_f):1.0})    
    
def sexsq(dae,syn_data,name):
    '''

    **Auxiliar equations**

    .. math::
        :nowrap:
        
        \begin{eqnarray}
            v_d &=& V \sin(\delta - \theta) \\
            v_q &=& V \cos(\delta - \theta) \\
            p_e &=& i_d \left(v_d + R_a i_d\right) + i_q \left(v_q + R_a i_q\right)  \\   
            \omega_s &=& \omega_{coi}
        \end{eqnarray} 


    **Dynamic equations**

    .. math::
        :nowrap:
        
        \begin{eqnarray}
            \frac{ d\delta}{dt} &=& \Omega_b \left(\omega - \omega_s \right) - K_{\delta} \delta  \\
            \frac{ d\omega}{dt} &=& \frac{1}{2H} \left(p_m - p_e - D  \left(\omega - \omega_s \right) \right)  \\
            \frac{ de'_q}{dt} &=& \frac{1}{T'_{d0}} \left(-e'_q - \left(X_d - X'_d\right) i_d + v_f\right)  \\
            \frac{ de'_d}{dt} &=& \frac{1}{T'_{q0}} \left(-e'_d + \left(X_q - X'_q\right) i_q\right)
        \end{eqnarray} 


    **Algebraic equations**
    

    .. math::
        :nowrap:
        
        \begin{eqnarray}
             0 &=& - v_{f } + \\begin{cases} V_{min } & \\text{for}\\: V_{min } > K_{a } \\left(- v_{c } + v_{pss } + v_{ref }\\right) + K_{ai } \\xi_{v } \\\\V_{max } & \\text{for}\\: V_{max } < K_{a } \\left(- v_{c } + v_{pss } + v_{ref }\\right) + K_{ai } \\xi_{v } \\\\K_{a } \\left(- v_{c } + v_{pss } + v_{ref }\\right) + K_{ai } \\xi_{v } & \\text{otherwise} \\end{cases} \\
             0  &=& v_q + R_a i_q + X'_d i_d - e'_q \\
             0  &=& v_d + R_a i_d - X'_q i_q - e'_d \\
             0  &=& i_d v_d + i_q v_q - p_g  \\
             0  &=& i_d v_q - i_q v_d - q_g 
        \end{eqnarray} 

    .. table:: Constants
        :widths: auto

        ================== =========== ============================================= =========== 
        Variable           Code        Description                                   Units
        ================== =========== ============================================= ===========
        :math:`S_n`        ``S_n``     Nominal power                                  VA
        :math:`H`          ``H``       Inertia constaant                              s
        :math:`S_n`        ``S_n``     Nominal power                                  VA
        :math:`D`          ``D``       Damping coefficient                            s
        :math:`X_q`        ``X_q``     q-axis synchronous reactance                   pu-m
        :math:`X'_q`       ``X1q``     q-axis transient reactance                     pu-m
        :math:`T'_{q0}`    ``T1q0``    q-axis open circuit transient time constant    s  
        :math:`X_d`        ``X_d``     d-axis synchronous reactance                   pu-m  
        :math:`X'_d`       ``X1d``     d-axis transient reactance                     pu-m
        :math:`T'_{d0}`    ``T1d0``    d-axis open circuit transient time constant    s
        :math:`R_a`        ``R_a``     Armature resistance                            pu-m
        :math:`K_{\delta}` ``K_delta`` Reference machine constant                     -
        :math:`K_{sec}`    ``K_sec``   Secondary frequency control participation      -
        ================== =========== ============================================= ===========

    .. table:: Dynamic states
        :widths: auto

        ================= =========== ============================================= =========== 
        Variable          Code        Description                                   Units
        ================= =========== ============================================= ===========
        :math:`\delta`    ``delta``    Rotor angle                                  rad
        :math:`\omega`    ``omega``    Rotor speed                                  pu
        :math:`e'_q`      ``e1q``      q-axis transient voltage                     pu
        :math:`e'_d`      ``e1d``      d-axis transient voltage                     pu
        ================= =========== ============================================= ===========


    .. table:: Algebraic states
        :widths: auto

        ================= =========== ============================================= =========== 
        Variable          Code        Description                                   Units
        ================= =========== ============================================= ===========
        :math:`i_d`       ``i_d``      d-axis current                               pu-m
        :math:`i_q`       ``i_q``      q-axis current                               pu-m
        :math:`p_g`       ``p_g``      Active power                                 pu-m
        :math:`q_g`       ``q_g``      Reactive power                               pu-m
        ================= =========== ============================================= ===========

    .. table:: Inputs
        :widths: auto

        ================= =========== ============================================= =========== 
        Variable          Code        Description                                   Units
        ================= =========== ============================================= ===========
        :math:`v_f`       ``v_f``      Field voltage                                pu-m
        :math:`p_m`       ``p_m``      Mechanical power                             pu-m
        ================= =========== ============================================= ===========
    
        T_r K_a K_aw
        v_ref  v_pss
        
        v_c
        v_f_nosat
        
     '''


    avr_data = syn_data['avr']
    
    v_t = sym.Symbol(f"V_{bus_name}", real=True)   
    q_g = sym.Symbol(f"q_g_{name}", real=True) 
    q = sym.Symbol(f"q_{name}", real=True)   


    v_c = sym.Symbol(f"v_c_{name}", real=True)  
    xi_vq  = sym.Symbol(f"xi_vq_{name}", real=True)
    v_f = sym.Symbol(f"v_f_{name}", real=True)  
    T_r = sym.Symbol(f"T_r_{name}", real=True) 
    K_a = sym.Symbol(f"K_a_{name}", real=True)
    K_ai = sym.Symbol(f"K_ai_{name}", real=True)
    V_min = sym.Symbol(f"V_min_{name}", real=True)
    V_max = sym.Symbol(f"V_max_{name}", real=True)
    K_aw = sym.Symbol(f"K_aw_{name}", real=True)   
    K_qv = sym.Symbol(f"K_qv_{name}", real=True)   

    
    v_ref = sym.Symbol(f"v_ref_{name}", real=True) 
    q_ref = sym.Symbol(f"q_ref_{name}", real=True) 
    v_pss = sym.Symbol(f"v_pss_{name}", real=True) 
    
    epsilon_v = v_ref - v_c + v_pss
    epsilon_q = q_ref - q
    epsilon_vq = (1-K_qv) * epsilon_v + K_qv*epsilon_q

    v_f_nosat = K_a*epsilon_vq + K_ai*xi_vq

    
    dv_c =   (v_t - v_c)/T_r
    dq   = (q_g - q)/T_r
    dxi_vq =   epsilon_vq  - K_aw*(v_f_nosat - v_f) 
    
    g_v_f  =   sym.Piecewise((V_min, v_f_nosat<V_min),(V_max,v_f_nosat>V_max),(v_f_nosat,True)) - v_f 
  #  g_v_f  =   v_f_nosat - v_f 
  
  
    
    dae['f'] += [dv_c,dq,dxi_vq]
    dae['x'] += [ v_c,q, xi_vq]
    dae['g'] += [g_v_f]
    dae['y_ini'] += [v_f] 
    dae['y_run'] += [v_f]  
    dae['params_dict'].update({str(K_a):avr_data['K_a']})
    dae['params_dict'].update({str(K_ai):avr_data['K_ai']})
    dae['params_dict'].update({str(T_r):avr_data['T_r']})  
    dae['params_dict'].update({str(V_min):avr_data['V_min']})  
    dae['params_dict'].update({str(V_max):avr_data['V_max']})  
    dae['params_dict'].update({str(K_aw):avr_data['K_aw']}) 
    dae['u_ini_dict'].update({str(v_ref):avr_data['v_ref']})
    dae['u_ini_dict'].update({str(q_ref):0.0})
    dae['u_ini_dict'].update({str(v_pss):avr_data['v_pss']})
    dae['u_run_dict'].update({str(v_ref):avr_data['v_ref']})
    dae['u_run_dict'].update({str(q_ref):0.0})

    dae['u_run_dict'].update({str(v_pss):avr_data['v_pss']})
    dae['params_dict'].update({str(K_qv):0.0})

    dae['xy_0_dict'].update({str(xi_vq):1})


def test():
    import numpy as np
    import sympy as sym
    import hjson
    from pydae.bmapu.bmapu_builder import bmapu
    import pydae.build_cffi as db
    import pytest

    grid = bmapu('sexs.hjson')
    grid.checker()
    grid.uz_jacs = True
    grid.build('temp')

    import temp

    model = temp.model()

    v_ref_1 = 1.05
    model.ini({'p_m_1':0.5,'v_ref_1':v_ref_1},'xy_0.json')

    model.report_x()
    model.report_y()

    assert model.get_value('V_1') == pytest.approx(v_ref_1, rel=0.001)
    # assert model.get_value('q_A2') == pytest.approx(-q_ref, rel=0.05)

    model.ini({'p_m_1':0.5,'v_ref_1':1.0},'xy_0.json')
    model.run(1.0,{})
    model.run(15.0,{'v_ref_1':1.05})
    model.post()

    import matplotlib.pyplot as plt

    

    fig,axes = plt.subplots()
    axes.plot(model.Time,model.get_values('V_1'))
    fig.savefig('sexs_step.svg')



    # assert model.get_value('p_dc_A1') == pytest.approx(p_mp, rel=0.01)
    # assert model.get_value('p_A2') == pytest.approx( -p_mp, rel=0.1)
    # assert model.get_value('q_A2') == pytest.approx(-q_ref, rel=0.05)


if __name__ == '__main__':

    #development()
    test()


# -*- coding: utf-8 -*-
"""
Created on Thu August 10 23:52:55 2022

@author: jmmauricio
"""

import numpy as np
import sympy as sym

def pv_dq_d(grid,name,bus_name,data_dict):
    '''

    VSC model with L filter coupling and purely algebraic.
    PQ control is implemented. 

    parameters
    ----------

    S_n: nominal power in VA
    U_n: nominal rms phase to phase voltage in V
    F_n: nominal frequency in Hz
    X_s: coupling reactance in pu (base machine S_n)
    R_s: coupling resistance in pu (base machine S_n)

    inputs
    ------

    p_s_ref: active power reference (pu, S_n base)
    q_s_ref: reactive power reference (pu, S_n base)
    v_dc: dc voltage in pu (when v_dc = 1 and m = 1, v_ac = 1)

    example
    -------

    "vscs": [{"bus":bus_name,"type":"pv_pq",
                 "S_n":1e6,"U_n":400.0,"F_n":50.0,
                 "X_s":0.1,"R_s":0.01,"monitor":True,
                 "I_sc":3.87,"V_oc":42.1,"I_mp":3.56,"V_mp":33.7,
                 "K_vt":-0.160,"K_it":0.065,
                 "N_pv_s":25,"N_pv_p":250}]
    
    '''

    sin = sym.sin
    cos = sym.cos

    ## Common
    ### inputs
    V_s = sym.Symbol(f"V_{bus_name}", real=True)
    p_s = sym.Symbol(f"p_s_{name}", real=True)
    theta_s = sym.Symbol(f"theta_{bus_name}", real=True)
    v_dc = sym.Symbol(f"v_dc_{name}", real=True)
    p_s_ppc = sym.Symbol(f"p_s_ppc_{name}", real=True)
    q_s_ppc = sym.Symbol(f"q_s_ppc_{name}", real=True)

    ### parameters
    S_n = sym.Symbol(f"S_n_{name}", real=True)
    U_n = sym.Symbol(f"U_n_{name}", real=True)
    V_dc_b = U_n*np.sqrt(2)
    X_s = sym.Symbol(f"X_s_{name}", real=True)
    R_s = sym.Symbol(f"R_s_{name}", real=True)

    ## PV
    K_vt,K_it = sym.symbols(f"K_vt_{name},K_it_{name}", real=True)
    V_oc,V_mp,I_sc,I_mp = sym.symbols(f"V_oc_{name},V_mp_{name},I_sc_{name},I_mp_{name}", real=True)
    temp_deg,irrad = sym.symbols(f"temp_deg_{name},irrad_{name}", real=True)
    T_stc_k,i,v = sym.symbols(f"T_stc_k_{name},i_{name},v_{name}", real=True)
    v_dc,v_dc_v,K_it = sym.symbols(f"v_dc_{name},v_dc_v_{name},K_it_{name}", real=True)
    N_pv_s,N_pv_p = sym.symbols(f"N_pv_s_{name},N_pv_p_{name}", real=True)

    T_stc_deg = 25.0

    V_oc_t = N_pv_s*V_oc * (1 + K_vt/100.0*(temp_deg - T_stc_deg))
    V_mp_t = N_pv_s*V_mp * (1 + K_vt/100.0*(temp_deg - T_stc_deg))
    I_sc_t = N_pv_p*I_sc * (1 + K_it/100.0*(temp_deg - T_stc_deg))
    I_mp_t = N_pv_p*I_mp * (1 + K_it/100.0*(temp_deg - T_stc_deg))
    I_mp_i = I_mp_t*irrad/1000.0

    v_1,i_1 = V_mp_t,I_mp_i
    v_2,i_2 = V_oc_t,0

    # (v_1 - v)/(v_1 - v_2) = (i_1 - i)/(i_1 - i_2)
    i_pv = p_s*S_n/(v_dc*V_dc_b)
    p_mp = (V_mp_t*I_mp_i)/S_n
    #v_dc_v = v_1 - (i_1 - i_pv)*(v_1 - v_2)/(i_1 - i_2) 
    v_dc_v =  v_1 - (i_1 - i_pv)*(v_1 - v_2)/(i_1 - i_2) 
    g_v_dc = -v_dc + v_dc_v/V_dc_b

    grid.dae['g'] +=     [g_v_dc]
    grid.dae['y_ini'] += [  v_dc]  
    grid.dae['y_run'] += [  v_dc]  

    grid.dae['u_ini_dict'].update({f'{str(irrad)}':1000.0})
    grid.dae['u_run_dict'].update({f'{str(irrad)}':1000.0})

    grid.dae['u_ini_dict'].update({f'{str(temp_deg)}':25.0})
    grid.dae['u_run_dict'].update({f'{str(temp_deg)}':25.0})

    grid.dae['params_dict'].update({
                   str(I_sc):data_dict['I_sc'],
                   str(I_mp):data_dict['I_mp'],
                   str(V_mp):data_dict['V_mp'],
                   str(V_oc):data_dict['V_oc'],
                   str(N_pv_s):data_dict['N_pv_s'],
                   str(N_pv_p):data_dict['N_pv_p'],
                   str(K_vt):data_dict['K_vt'],
                   str(K_it):data_dict['K_it']
                   })

   
    
    #grid.dae['xy_0_dict'].update({f"v_dc_v_{name}":data_dict['V_mp']*data_dict['N_pv_s']})
    grid.dae['xy_0_dict'].update({f"v_dc_{name}":1.5})

    ## VSC control 
    ### inputs
    p_s_ref = sym.Symbol(f"p_s_ref_{name}", real=True)
    q_s_ref = sym.Symbol(f"q_s_ref_{name}", real=True)
    i_sa_ref = sym.Symbol(f"i_sa_ref_{name}", real=True)
    i_sr_ref = sym.Symbol(f"i_sr_ref_{name}", real=True)

    ### dynamic states

    ### algebraic states
    i_sd_pq_ref,i_sq_pq_ref = sym.symbols(f'i_sd_pq_ref_{name},i_sq_pq_ref_{name}', real=True)
    i_sd_ar_ref,i_sq_ar_ref = sym.symbols(f'i_sd_ar_ref_{name},i_sq_ar_ref_{name}', real=True)
    i_sd_ref,i_sq_ref = sym.symbols(f'i_sd_ref_{name},i_sq_ref_{name}', real=True)
    i_sd_ref,i_sq_ref = sym.symbols(f'i_sd_ref_{name},i_sq_ref_{name}', real=True)

    v_td_ref,v_tq_ref = sym.symbols(f'v_td_ref_{name},v_tq_ref_{name}', real=True)
    v_lvrt,lvrt_ext = sym.symbols(f"v_lvrt_{name},lvrt_ext_{name}", real=True)

    x_p_lp1, x_p_lp2 = sym.symbols(f"x_p_lp1_{name},x_p_lp2_{name}", real=True)
    x_q_lp1, x_q_lp2 = sym.symbols(f"x_q_lp1_{name},x_q_lp2_{name}", real=True) 
    T_lp1p, T_lp2p = sym.symbols(f"T_lp1p_{name},T_lp2p_{name}", real=True) 
    T_lp1q, T_lp2q = sym.symbols(f"T_lp1q_{name},T_lp2q_{name}", real=True) 
    PRampUp,PRampDown, QRampUp, QRampDown, ramp_enable = sym.symbols(f"PRampUp_{name},PRampDown_{name},QRampUp_{name},QRampDown_{name},ramp_enable_{name}", real=True) 

    ### parameters
    
    ### auxiliar
    delta = theta_s # ideal PLL
    v_sD = V_s*sin(theta_s)  # v_si   e^(-j)
    v_sQ = V_s*cos(theta_s)  # v_sr
    v_sd = v_sD * cos(delta) - v_sQ * sin(delta)   
    v_sq = v_sD * sin(delta) + v_sQ * cos(delta)

    v_m = sym.sqrt(v_sd**2 + v_sq**2)
    lvrt = sym.Piecewise((0.0,v_m>=v_lvrt),(1.0,v_m<v_lvrt)) + lvrt_ext

    p_s_ppc_d = x_p_lp2
    q_s_ppc_d = x_q_lp2

    p_s_ref = sym.Piecewise((p_s_ppc_d,p_s_ppc_d<p_mp),(p_mp,p_s_ppc_d>=p_mp))
    q_s_ref = q_s_ppc_d

    dx_p_lp1_nosat = ramp_enable/T_lp1p*(p_s_ppc  - x_p_lp1)
    dx_p_lp1 = sym.Piecewise((PRampDown,dx_p_lp1_nosat<PRampDown),(PRampUp,dx_p_lp1_nosat>PRampUp),(dx_p_lp1_nosat,True))
    dx_p_lp2 = 1/T_lp2p*(x_p_lp1 - x_p_lp2)
    
    dx_q_lp1_nosat = ramp_enable/T_lp1q*(q_s_ppc  - x_q_lp1)
    dx_q_lp1 =sym.Piecewise((QRampDown,dx_q_lp1_nosat<QRampDown),(QRampUp,dx_q_lp1_nosat>QRampUp),(dx_q_lp1_nosat,True))
    dx_q_lp2 = 1/T_lp2q*(x_q_lp1 - x_q_lp2)

    ### dynamic equations            
    grid.dae['f'] += [dx_p_lp1,dx_p_lp2,dx_q_lp1,dx_q_lp2]
    grid.dae['x'] += [ x_p_lp1, x_p_lp2, x_q_lp1, x_q_lp2] 
    grid.dae['xy_0_dict'].update({f"x_p_lp1_{name}":1.5,f"x_p_lp2_{name}":1.5})
    grid.dae['xy_0_dict'].update({f"x_q_lp1_{name}":0.0,f"x_q_lp2_{name}":0.0})
      
    ### algebraic equations   
    #g_i_sd_pq_ref  = i_sd_pq_ref*v_sd + i_sq_pq_ref*v_sq - p_s_ref  
    #g_i_sq_pq_ref  =-i_sq_pq_ref*v_sd + i_sd_pq_ref*v_sq - q_s_ref 
    #g_i_sd_ar_ref  = i_sd_ar_ref*v_sd/v_m + i_sq_ar_ref*v_sq/v_m - i_sa_ref  
    #g_i_sq_ar_ref  =-i_sq_ar_ref*v_sd/v_m + i_sd_ar_ref*v_sq/v_m - i_sr_ref 
    #g_v_td_ref  = v_td_ref - R_s*i_sd_ref + X_s*i_sq_ref - v_sd  
    #g_v_tq_ref  = v_tq_ref - R_s*i_sq_ref - X_s*i_sd_ref - v_sq 

    i_sd_ar_ref = i_sa_ref*v_sd/sym.sqrt(v_sd**2 + v_sq**2) + i_sr_ref*v_sq/sym.sqrt(v_sd**2 + v_sq**2) 
    i_sq_ar_ref = i_sa_ref*v_sq/sym.sqrt(v_sd**2 + v_sq**2) - i_sr_ref*v_sd/sym.sqrt(v_sd**2 + v_sq**2)

    i_sd_pq_ref = (p_s_ref*v_sd + q_s_ref*v_sq)/(v_sd**2 + v_sq**2)
    i_sq_pq_ref = (p_s_ref*v_sq - q_s_ref*v_sd)/(v_sd**2 + v_sq**2)
    i_sd_ref_nosat = (1.0-lvrt)*i_sd_pq_ref + lvrt*i_sd_ar_ref
    i_sq_ref_nosat = (1.0-lvrt)*i_sq_pq_ref + lvrt*i_sq_ar_ref
    g_i_sd_ref = -i_sd_ref + sym.Piecewise((-1.2,i_sd_ref_nosat<-1.2),(1.2,i_sd_ref_nosat>1.2),(i_sd_ref_nosat,True))
    g_i_sq_ref = -i_sq_ref + sym.Piecewise((-1.2,i_sq_ref_nosat<-1.2),(1.2,i_sq_ref_nosat>1.2),(i_sq_ref_nosat,True))

    v_td_ref  =  R_s*i_sd_ref - X_s*i_sq_ref + v_sd  
    v_tq_ref  =  R_s*i_sq_ref + X_s*i_sd_ref + v_sq 

    v_tD_ref = v_td_ref * cos(delta) + v_tq_ref * sin(delta)   
    v_tQ_ref =-v_td_ref * sin(delta) + v_tq_ref * cos(delta)    
    v_ti_ref = v_tD_ref
    v_tr_ref = v_tQ_ref 
    m_ref = sym.sqrt(v_tr_ref**2 + v_ti_ref**2)/v_dc
    theta_t_ref = sym.atan2(v_ti_ref,v_tr_ref) 


    ### dae 
    grid.dae['g'] += [g_i_sd_ref, g_i_sq_ref]
    grid.dae['y_ini'] += [i_sq_ref, i_sd_ref]  
    grid.dae['y_run'] += [i_sq_ref, i_sd_ref]  

    grid.dae['u_ini_dict'].update({f'lvrt_ext_{name}':0.0})
    grid.dae['u_run_dict'].update({f'lvrt_ext_{name}':0.0})

    grid.dae['u_ini_dict'].update({f'ramp_enable_{name}':1e-5})
    grid.dae['u_run_dict'].update({f'ramp_enable_{name}':1.0})

    grid.dae['u_ini_dict'].update({f'p_s_ppc_{name}':1.5})
    grid.dae['u_run_dict'].update({f'p_s_ppc_{name}':1.5})

    grid.dae['u_ini_dict'].update({f'q_s_ppc_{name}':0.0})
    grid.dae['u_run_dict'].update({f'q_s_ppc_{name}':0.0})

    grid.dae['u_ini_dict'].update({f'{str(i_sa_ref)}':0.0})
    grid.dae['u_run_dict'].update({f'{str(i_sa_ref)}':0.0})

    grid.dae['u_ini_dict'].update({f'{str(i_sr_ref)}':0.0})
    grid.dae['u_run_dict'].update({f'{str(i_sr_ref)}':0.0})

    grid.dae['params_dict'].update({f'{str(v_lvrt)}':0.8})
    grid.dae['params_dict'].update({f'{str(T_lp1p)}':0.1,f'{str(T_lp2p)}':0.1})
    grid.dae['params_dict'].update({f'{str(T_lp1q)}':0.1,f'{str(T_lp2q)}':0.1})
    grid.dae['params_dict'].update({f'{str(PRampUp)}':2.5,f'{str(PRampDown)}':-2.5})
    grid.dae['params_dict'].update({f'{str(QRampUp)}':2.5,f'{str(QRampDown)}':-2.5})

    ### outputs
    grid.dae['h_dict'].update({f"m_ref_{name}":m_ref})
    grid.dae['h_dict'].update({f"v_sd_{name}":v_sd})
    grid.dae['h_dict'].update({f"v_sq_{name}":v_sq})
    grid.dae['h_dict'].update({f"lvrt_{name}":lvrt})


    ## VSC model 
    # m = sym.Symbol(f"m_{name}", real=True)
    # theta_t = sym.Symbol(f"theta_t_{name}", real=True)
      
    ### dynamic states
    #m_f = sym.Symbol(f"m_f_{name}", real=True)

    ### algebraic states
    i_si = sym.Symbol(f"i_si_{name}", real=True)
    i_sr = sym.Symbol(f"i_sr_{name}", real=True)       
    p_s = sym.Symbol(f"p_s_{name}", real=True)
    q_s = sym.Symbol(f"q_s_{name}", real=True)

    ### parameters
    F_n = sym.Symbol(f"F_n_{name}", real=True)            

    
    params_list = ['S_n','F_n','U_n','X_s','R_s']
    
    ### auxiliar
    v_si = V_s*sin(theta_s)  # v_D, e^(-j)
    v_sr = V_s*cos(theta_s)  # v_Q
    Omega_b = 2*np.pi*F_n
    m = m_ref
    theta_t = theta_t_ref
    v_t_m = m*v_dc
    v_tr = v_t_m*cos(theta_t)
    v_ti = v_t_m*sin(theta_t)
    
    ### dynamic equations            

    ### algebraic equations   
    v_ti = v_ti_ref 
    v_tr = v_tr_ref 
    g_i_si = v_ti - R_s*i_si + X_s*i_sr - v_si  
    g_i_sr = v_tr - R_s*i_sr - X_s*i_si - v_sr 
    # i_sr = (-R_s*v_sr + R_s*v_tr + X_s*v_si - X_s*v_ti)/(R_s**2 + X_s**2)
    # i_si = (-R_s*v_si + R_s*v_ti - X_s*v_sr + X_s*v_tr)/(R_s**2 + X_s**2)
    g_p_s  = i_si*v_si + i_sr*v_sr - p_s  
    g_q_s  = i_si*v_sr - i_sr*v_si - q_s 

    ### dae 
    f_vsg = []
    x_vsg = []
    g_vsg = [g_i_si,g_i_sr,g_p_s,g_q_s]
    y_vsg = [  i_sr,  i_si,  p_s,  q_s]

    grid.dae['f'] += f_vsg
    grid.dae['x'] += x_vsg
    grid.dae['g'] += g_vsg
    grid.dae['y_ini'] += y_vsg  
    grid.dae['y_run'] += y_vsg  
    
    # grid.dae['u_ini_dict'].update({f'{m}':1.0})
    # grid.dae['u_run_dict'].update({f'{m}':1.0})

    # grid.dae['u_ini_dict'].update({f'{theta_t}':0.0})
    # grid.dae['u_run_dict'].update({f'{theta_t}':0.0})

    # grid.dae['u_ini_dict'].update({f'{v_dc}':1.2})
    # grid.dae['u_run_dict'].update({f'{v_dc}':1.2})

    grid.dae['xy_0_dict'].update({str(p_s):0.5})
       
    ### outputs


    for item in params_list:       
        grid.dae['params_dict'].update({f"{item}_{name}":data_dict[item]}) 

    if 'monitor' in data_dict:
        if data_dict['monitor'] == True:
            
            grid.dae['h_dict'].update({f"v_dc_v_{name}":v_dc*V_dc_b})
            grid.dae['h_dict'].update({f"v_ac_v_{name}":v_t_m*U_n})
            grid.dae['h_dict'].update({f"v_dc_v_{name}":v_dc_v})    
            grid.dae['h_dict'].update({f"p_mp_{name}":p_mp})    
            grid.dae['h_dict'].update({f"i_pv_{name}":i_pv})    
            grid.dae['h_dict'].update({f"v_dc_{name}":v_dc}) 
            grid.dae['h_dict'].update({f"p_s_{name}":p_s})
            grid.dae['h_dict'].update({f"q_s_{name}":q_s})
            grid.dae['h_dict'].update({f"v_ti_{name}":v_ti})
            grid.dae['h_dict'].update({f"v_tr_{name}":v_tr})    
            grid.dae['h_dict'].update({f"i_si_{name}":i_si})
            grid.dae['h_dict'].update({f"i_sr_{name}":i_sr})
            i_s = sym.sqrt(i_sr**2 + i_si**2) 
            grid.dae['h_dict'].update({f"i_s_{name}":i_s})            

    p_W   = p_s * S_n
    q_var = q_s * S_n

    return p_W,q_var

def sym2model():
    v_sd = sym.Symbol(f"v_sd", real=True)
    v_sq = sym.Symbol(f"v_sq", real=True)
    p_s_ref = sym.Symbol(f"p_s_ref", real=True)
    q_s_ref = sym.Symbol(f"q_s_ref", real=True)
    i_sa_ref = sym.Symbol(f"i_sa_ref", real=True)
    i_sr_ref = sym.Symbol(f"i_sr_ref", real=True)
    i_sd_pq_ref,i_sq_pq_ref = sym.symbols(f'i_sd_pq_ref,i_sq_pq_ref', real=True)
    i_sd_ar_ref,i_sq_ar_ref = sym.symbols(f'i_sd_ar_ref,i_sq_ar_ref', real=True)


    g_i_sd_pq_ref  = i_sd_pq_ref*v_sd + i_sq_pq_ref*v_sq - p_s_ref  
    g_i_sq_pq_ref  =-i_sq_pq_ref*v_sd + i_sd_pq_ref*v_sq - q_s_ref 

    sol = sym.solve([g_i_sd_pq_ref,g_i_sq_pq_ref],[i_sd_pq_ref,i_sq_pq_ref])

    for item in sol:
        print(f"{item} = {sol[item]}")

    v_m = sym.sqrt(v_sd**2 + v_sq**2)
    g_i_sd_ar_ref  = i_sd_ar_ref*v_sd/v_m + i_sq_ar_ref*v_sq/v_m - i_sa_ref  
    g_i_sq_ar_ref  =-i_sq_ar_ref*v_sd/v_m + i_sd_ar_ref*v_sq/v_m - i_sr_ref 

    sol = sym.solve([g_i_sd_ar_ref,g_i_sq_ar_ref],[i_sd_ar_ref,i_sq_ar_ref])

    for item in sol:
        print(f"{item} = {sol[item]}")

    # coupling filter:
    v_tr,v_ti = sym.symbols(f"v_tr,v_ti", real=True)
    i_sr,i_si = sym.symbols(f"i_sr,i_si", real=True)
    v_sr,v_si = sym.symbols(f"v_sr,v_si", real=True)
    R_s,X_s =   sym.symbols(f"R_s,X_s", real=True)

    g_i_si = v_ti - R_s*i_si + X_s*i_sr - v_si  
    g_i_sr = v_tr - R_s*i_sr - X_s*i_si - v_sr 
    sol = sym.solve([g_i_sr,g_i_si],[i_sr,i_si])

    for item in sol:
        print(f"{item} = {sol[item]}")

    # simplified PV module




#sym2model()
if __name__ == "__main__":

    from pydae.bmapu import bmapu_builder
    
    data = {
    "system":{"name":"test_model","S_base":100e6, "K_p_agc":0.0,"K_i_agc":0.0,"K_xif":0.01},       
    "buses":[{"name":"1", "P_W":0.0,"Q_var":0.0,"U_kV":20.0},
             {"name":"2", "P_W":0.0,"Q_var":0.0,"U_kV":20.0}
            ],
    "lines":[{"bus_j":"1", "bus_k":"2", "X_pu":0.05,"R_pu":0.0,"Bs_pu":0.0,"S_mva":10000.0}],
    "genapes":[{"bus":"2","S_n":10000e6,"F_n":50.0,"X_v":0.001,"R_v":0.0,"K_delta":0.001,"K_alpha":1e-6}]
    }

    grid = bmapu_builder.bmapu(data)
    #grid.checker()
    grid.uz_jacs = True
    grid.verbose = False
    grid.construct('test_model')

    bus_name = "1"
    data_dict = {"bus":bus_name,"type":"pv_pq",
                 "S_n":1e6,"U_n":400.0,"F_n":50.0,
                 "X_s":0.1,"R_s":0.01,"monitor":True,
                 "I_sc":8,"V_oc":42.1,"I_mp":3.56,"V_mp":33.7,
                 "K_vt":-0.160,"K_it":0.065,
                 "N_pv_s":25,"N_pv_p":250}

    p_W,q_var = pv_dq(grid,'1','1',data_dict)

    # grid power injection
    idx_bus = [bus['name'] for bus in grid.data['buses']].index(bus_name) # get the number of the bus where the syn is connected

    S_base = sym.Symbol('S_base', real = True)
    grid.dae['g'][idx_bus*2]   += -p_W/S_base
    grid.dae['g'][idx_bus*2+1] += -q_var/S_base
    
    grid.compile()   

    import test_model

    xy_0 = {
    "V_1": 1.0,
    "theta_1": 0.0,
    "V_2": 1.0,
    "theta_2": 0.0,
    "omega_coi": 1.0,
    "omega_2": 1.0,
    "v_dc_v_1":800
    }
    model = test_model.model()
    model.ini({'p_s_ppc_1':0.9,'q_s_ppc_1':0.2,'irrad_1':100,'temp_deg_1':75,
               'v_ref_2':1.0},xy_0)

    #model.report_y()
    model.report_z()
    #model.report_u()



# -*- coding: utf-8 -*-
"""
Created on Thu August 10 23:52:55 2022

@author: jmmauricio
"""

from pydae.bmapu.govs.tgov1 import tgov1
from pydae.bmapu.govs.agov1 import agov1
from pydae.bmapu.govs.ntsieeeg1 import ntsieeeg1

def add_gov(dae,syn_data,name,bus_name):
    
    if syn_data['gov']['type'] == 'tgov1':
        tgov1(dae,syn_data,name,bus_name)
    if syn_data['gov']['type'] == 'agov1':
        agov1(dae,syn_data,name,bus_name)
    if syn_data['gov']['type'] == 'ntsieeeg1':
        ntsieeeg1(dae,syn_data,name,bus_name)


import numpy as np
import matplotlib.pyplot as plt
import ipywidgets
plt.style.use('https://raw.githubusercontent.com/pydae/pydae/master/src/pydae/edashboards/presentation.mplstyle')
colors = plt.rcParams['axes.prop_cycle'].by_key()['color']
import sympy as sym
import pydae.build_cffi as db

class dashboard():
    
    def __init__(self):

        self.colors = colors
        
        # model.Dt = 0.01
        # model.decimation = 1
        # model.ini({'v_ref_1':1.0,'p_m_1':0.0},'xy_0.json')
        # model.run(30.0,{})
        # model.post();
        
        # self.model = model
        
        # self.colors = plt.rcParams['axes.prop_cycle'].by_key()['color']
        # self.widgets()

    def build(self):

        u_12  = sym.Symbol(f"u_12", real=True)
        timer  = sym.Symbol(f"timer", real=True)

        x_12  = sym.Symbol(f"x_12", real=True)
        x_t  = sym.Symbol(f"x_t", real=True)

        t_1  = sym.Symbol(f"t_1", real=True)

        T_1 = sym.Symbol(f"T_1", real=True) 
        T_2 = sym.Symbol(f"T_2", real=True)
        omega = sym.Symbol(f"omega", real=True)
        K_sin = sym.Symbol(f"K_sin", real=True)


        u = u_12 + K_sin*sym.sin(omega*t_1)
        z_12 = (u - x_12)*T_1/T_2 + x_12
        dx_12 =  (u - x_12)/T_2      # lead compensator state
        dt_1  = timer - (1-timer)*t_1
        
        params_dict = {'T_1':1.0,'T_2':1.0, 'omega':2*np.pi*10, 'K_sin':0.0}

        sys_dict = {'name':'ctrl_leadlag',
            'params_dict':params_dict,
            'f_list':[dx_12, dt_1],
            'g_list':[],
            'x_list':[ x_12, t_1],
            'y_ini_list':[],
            'y_run_list':[],
            'u_ini_dict':{'u_12':0.0,'timer':0.0},
            'u_run_dict':{'u_12':0.0,'timer':0.0},
            'h_dict':{'z':z_12, 'u':u, 't_1':t_1}
        }

        bldr = db.builder(sys_dict)
        bldr.build()

    def start(self):
        import ctrl_leadlag 
        self.model = ctrl_leadlag.model()
        self.T_1 = 1.0
        self.T_2 = 1.0
        self.omega = 1.0
        self.amplitude = 1.0

    def simulate(self):

        self.model.ini({'timer':0.0,'T_1':self.T_1,'T_2':self.T_2},'xy_0.json')
        self.model.run(1,{})
        self.model.run(10,{'timer':1.0})

        self.model.post()
      
    def widgets(self):
        
        self.simulate()
        model = self.model
        colors = self.colors

        plt.ioff()
        plt.clf()


        fig, axes = plt.subplots(nrows=1, figsize=(9, 4), frameon=False)
        fig.canvas.toolbar_visible = False

        self.line_u = axes.plot(model.Time, model.get_values('u'), label='$u$', color=colors[0])
        self.line_z = axes.plot(model.Time, model.get_values('z'), label='$z$', color=colors[1])
        # self.line_omega = axes[1,0].plot(model.Time, model.get_values('omega_1'), label='$\sf \omega$', color=colors[1])
        # self.line_v_1 = axes[0,1].plot(model.Time, model.get_values('V_1'), label='$\sf V_1$', color=colors[5])
        # #line_theta_1 = axes[0,1].plot(T, Y[:,syst.y_list.index('theta_1')], label='$\sf \\theta_1$')
        # self.line_p_t = axes[1,1].plot(model.Time, model.get_values('p_g_1'), label='$\sf P_t$', color=colors[2])
        # self.line_q_t = axes[1,1].plot(model.Time, model.get_values('q_g_1'), label='$\sf Q_t$', color=colors[0])

        # y_labels = ['$\delta$','$\omega$','$P_t$']

        # axes[0,0].set_ylim((-1,2))
        # axes[1,0].set_ylim((0.95,1.05))
        # axes[0,1].set_ylim((0.8,1.2))
        # axes[1,1].set_ylim((-0.5,1.5))

        # axes[0,0].grid(True)
        # axes[1,0].grid(True)
        # axes[0,1].grid(True)
        # axes[1,1].grid(True)
        # axes[0,0].legend(loc='best')
        # axes[1,0].legend(loc='best')
        # axes[0,1].legend(loc='best')
        # axes[1,1].legend(loc='best')

        # axes[1,0].set_xlabel('Time (s)')  
        # axes[1,1].set_xlabel('Time (s)') 

        fig.tight_layout()
        
        self.fig = fig
        #axes[0].set_title('Par en función de la velocidad')
        #axes[1].set_title('Corriente en función de la velocidad')

        fig.savefig('out.svg')
        self.sld_T_1 = ipywidgets.FloatSlider(orientation='horizontal',description = "T1", 
                                        value=self.T_1, min=0.0,max= 2.0, 
                                        step=.1)

        self.sld_T_2 = ipywidgets.FloatSlider(orientation='horizontal',description = "T2", 
                                        value=self.T_2, min=0.0,max= 2.0, 
                                        step=.1)

        self.sld_omega = ipywidgets.FloatSlider(orientation='horizontal',description = "omega", 
                                        value=self.omega, min=0.0,max= 10.0, 
                                        step=.1)
        self.sld_amplitude = ipywidgets.FloatSlider(orientation='horizontal',description = "Amplitude", 
                                        value=self.amplitude, min=0.0,max= 2.0, 
                                        step=.1)
        
        tab_leadlag = ipywidgets.VBox([self.sld_T_1,self.sld_T_2])
        tab_input = ipywidgets.VBox([self.sld_omega,self.sld_amplitude])

        self.tab = ipywidgets.Tab()
        self.tab.children = [tab_input,tab_leadlag]

        self.tab.set_title(0, 'Input')
        self.tab.set_title(1, 'Lead-Lag')

    def update(self,change):
        
        model = self.model

        self.simulate()

        
        T_1 = self.sld_T_1.value
        T_2 = self.sld_T_2.value

        self.line_u.set_data(model.Time, model.get_values('u'))
        self.line_z.set_data(model.Time, model.get_values('z'))



    def show(self):

        self.sld_T_1.observe(self.update, names='value')
        self.sld_T_2.observe(self.update, names='value')
        self.sld_omega.observe(self.update, names='value')
        self.sld_amplitude.observe(self.update, names='value')

        layout_row1 = ipywidgets.HBox([self.fig.canvas])
        layout_row2 = ipywidgets.HBox([self.tab])

        layout = ipywidgets.VBox([layout_row1,layout_row2])
        self.layout = layout
        display(self.layout)
     

if __name__ == "__main__":

    d = dashboard()
    d.build()
    d.start()
    d.widgets()
    
    

